<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\File;

class FSDirectories extends Command
{

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'fs:essential';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create essential FS directories in Salamanda.';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @param void
     * @return void
     */
    public function handle()
    {
        $dirs = [
            'storage' => ['/tmp', '/images/articles', '/images/thumbnails'],
            'others' => []
        ];

        foreach ($dirs as $loc => $dir) {
            if ($loc == 'storage') {
                // If storage is not linked, link it
                if (!realpath(public_path('storage'))) {
                    Artisan::call('storage:link --force');
                    $this->info('Symbolic link created successfully');
                }
                foreach ($dir as $path) {
                    File::ensureDirectoryExists(storage_path("app/public$path"));
                }
            } else if ($loc == 'others') {
                foreach ($dir as $path) {
                    File::ensureDirectoryExists($path);
                }
            }
        }
        $this->info('[SLM] Command executed successfully');
    }

}
