<?php

namespace App\Http\Controllers;

use App\Role;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rule;

class AdminController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function index()
    {
        return response()->json([
            'status' => 'success',
            'data' => User::with('department')->get()
        ]);
    }

    public function getCashierUsers()
    {
        return response()->json([
            'status' => 'success',
            'data' => User::whereHas('roles', function ($q) {
                $q->where('slug', 'cashier');
            })->get()
        ]);
    }

    public function createUser(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|string',
            'username' => 'required|unique:users,username',
            'email' => 'nullable|email|unique:users',
            'password' => 'required|string|min:6|confirmed',
            'department_id' => 'required'
        ]);

        $user = new User();
        $user->name = $request->input('name');
        $user->username = $request->input('username');
        $user->email = $request->input('email');
        $user->lang = 'en';
        $user->password = Hash::make($request->password);

        $user->active = $request->active;
        $user->creator_id = auth()->id();
        $user->department_id = $request->department_id;

        $user->save();
        $user->refresh();
        if ($request->role) {
            $user->roles()->attach([$request->role]);
        }
        return response()->json([
            'status' => 'success',
            'data' => User::with('department')->find($user->id)
        ]);
    }

    public function updateUser($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required|string',
            'username' => ['required', Rule::unique('users', 'username')->ignore($id)],
            'email' => ['nullable', 'email', Rule::unique('users', 'email')->ignore($id)],
            'department_id' => 'required'
        ]);

        $user = User::find($id);
        $user->name = $request->input('name');
        $user->username = $request->input('username');
        $user->email = $request->input('email');
        $user->department_id = $request->department_id;

        $user->update();

        return response()->json([
            'status' => 'success',
            'data' => User::with('department')->find($user->id)
        ]);
    }

    public function changeStatus($id)
    {
        $user = User::find($id);
        if ($user->id == auth()->id()) {
            return response()->json([
                'status' => 'error',
                'message' => 'You cannot deactivate yourself!!!'
            ], 403);
        }
        $user->active = !$user->active;
        $user->update();
        return response()->json([
            'status' => 'success',
            'data' => $user
        ]);
    }

    public function resetPassword($id, Request $request)
    {
        $data = $this->validate($request, [
            'password' => 'required|min:6'
        ]);
        $user = User::find($id);
        $user->password = Hash::make($data['password']);
        $user->update();
        return response()->json([
            'status' => 'success'
        ]);
    }
}
