<?php


namespace App\Http\Controllers;

//use http\Env\Request;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;


class AuthController extends Controller
{
    /**
     * Create a new AuthController instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth:api', ['except' => ['login', 'verifyPassword']]);
    }

    /**
     * Get a JWT via given credentials.
     *
     * @return JsonResponse
     */
    public function login()
    {
        $credentials = request(['username', 'password']);

        if (!$token = auth()->attempt($credentials)) {
            return response()->json(['error' => 'Incorrect username or password'], 401);
        }

        $user = auth()->user();
        if (!$user->active) {
            return response()->json(['error' => 'Your account has been suspended. Please, contact admin'], 401);
        }

        if (request()->input('app') === 'pos') {
            if (!$user->hasRole('cashier')) {
                return response()->json([
                    'error' => "ACCESS DENIED! PLEASE, CONTACT YOUR SYSTEM ADMINISTRATORS."
                ], 401);
            }
            if (!$user->session()->count()) {
                return response()->json([
                    'error' => "YOU DO NOT HAVE AN ACTIVE SESSION! PLEASE, CONTACT YOUR SYSTEM ADMINISTRATORS."
                ], 401);
            }
        }

        $copy = $user;
        $user->last_login = date('Y-m-d H:i:s');
        $user->update();

        return $this->respondWithToken($token, $copy);
    }

    /**
     * Get the authenticated User.
     *
     * @return JsonResponse
     */
    public function me()
    {
        return response()->json(auth()->user());
    }

    /**
     * Log the user out (Invalidate the token).
     *
     * @return JsonResponse
     */
    public function logout()
    {
        auth()->logout();

        return response()->json(['message' => 'Successfully logged out']);
    }

    /**
     * Refresh a token.
     *
     * @return JsonResponse
     */
    public function refresh()
    {
        return $this->respondWithToken(auth()->refresh(), auth()->user());
    }

    /**
     * Get the token array structure.
     *
     * @param string $token
     *
     * @param $user
     * @return JsonResponse
     */
    protected function respondWithToken($token, $user)
    {
        return response()->json([
            'status' => 'success',
            'token' => $token,
            'token_type' => 'bearer',
            'expires_in' => auth()->factory()->getTTL() * 60 * 60 * 24 * 5 * (request()->input('app') === 'pos' ? 12 : 1),
            'user' => $user
        ]);
    }

    public function changePassword(Request $request)
    {
        $data = $this->validate($request, [
            'old_password' => 'required',
            'password' => 'required|confirmed|min:6.'

        ]);
        $old_password = $data['old_password'];
        $user = Auth::user();

        if (Hash::check($old_password, $user->password)) {
            $user->password = Hash::make($data['password']);
            $user->update();
            return response()->json([
                'status' => 'success'
            ]);
        }
        return response()->json([
            'status' => 'error',
            'message' => "Unauthorized: Password match incorrect."
        ], 403);

    }

    public function verifyPassword(): JsonResponse
    {
        $credentials = request(['username', 'password']);
        if (!auth()->attempt($credentials)) {
            return response()->json(['error' => 'Incorrect password'], 401);
        }
        return $this->refresh();
    }

    public function changeLanguage(Request $request)
    {
        $data = $this->validate($request, [
            'lang' => 'required|string'
        ]);
        $user = Auth::user();
        $user->lang = $request->input('lang');
        $user->update();
        return response()->json([
            'status' => 'success'
        ]);

    }

}
