<?php

namespace App\Http\Controllers;

use App\Config;
use Validator;
use App\User;
use Firebase\JWT\JWT;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Laravel\Lumen\Routing\Controller as BaseController;
class AuthSalesController extends BaseController
{
    /**
     * The request instance.
     *
     * @var \Illuminate\Http\Request
     */
    private $request;
    /**
     * Create a new controller instance.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return void
     */
    public function __construct(Request $request) {
    	$this->request = $request;
    }
    /**
     * Create a new token.
     *
     * @param  \App\User   $user
     * @return string
     */
    protected function jwt(User $user) {
    	$payload = [
            'iss' => "lumen-jwt", // Issuer of the token
            'sub' => $user->id, // Subject of the token
            'sub_id' => $user->id,
            'iat' => time(), // Time when JWT was issued.
            'exp' => time() + 60*60*60 // Expiration time
        ];

        // As you can see we are passing `JWT_SECRET` as the second parameter that will
        // be used to decode the token in the future.
        return JWT::encode($payload, env('JWT_SECRET'));
    }
    /**
     * Authenticate a user and return the token if the provided credentials are correct.
     *
     * @param  \App\User   $user
     * @return mixed
     */
    public function authenticate(User $user, Request $request) {

    	$this->validate($this->request, [
    		'username'     => 'required',
    		'password'  => 'required'
    	]);
        // Find the user by email
    	$user = User::where([['username', $this->request->input('username')]])->first();
        // return $user;
    	if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the
            // below respose for now.
    		return response()->json([
    			'error' => 'Username does not exist.'
    		], 400);
    	}

        if (!$user->is_admin && !$user->session) {
            return response()->json([
                'error' => "You've no available session. Contact administrator to create one for you"
            ], 400);
        }

        if (!$user->status) {
            return response()->json([
                'error' => 'Account deactivated! Contact Administrator'
            ], 400);
        }
        // Verify the password and generate the token
    	if (Hash::check($this->request->input('password'), $user->password)) {
            $user->last_login = date('Y-m-d H:i:s');
            $user->update();
    		return response()->json([
    			'token' => $this->jwt($user),
                'department' => $user->department,
                'user' => $user,
                'license' => Config::latest()->whereActive(true)->first()
            ], 200);
    	}
        // Bad Request response
    	return response()->json([
    		'error' => 'Incorrect username or password.'
    	], 400);
    }

    public function regenerateToken(Request $request)
    {
    	$user = $request->user();
    	return response()->json([
    		'user' => $user,
            'cong' => $user->cong,
            'token' => $this->jwt($user)
        ]);
    }
}
