<?php

namespace App\Http\Controllers;

use App\Category;
use App\Deposit;
use App\Account;
use App\AccountCategory;
use App\Item;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Illuminate\Validation\Rule;
use Illuminate\Validation\ValidationException;

class CategoriesController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function categories(Request $request)
    {
        return response()->json([
            'status' => 'success',
            'data' => Category::with(['user'])->latest()->get()
        ]);
    }

    public function index(Request $request)
    {
        $cats = Category::latest()->with(['items'])->get();

        return response()->json([
            'status' => 'success',
            'data' => $cats
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws ValidationException
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => ['required', 'unique:categories,name'],
            // 'department' => ['required', 'numeric']
        ]);
        $category = new Category();
        $category->name = $request->input('name');
        if (Schema::hasColumn("categories", "colour") && $request->filled('colour')) {
            $category->colour = $request->input('colour');
        }
        // @todo Make department_id dynamic. This enables filtering categories per department or store
        $user = $request->user();
        if ($user->id) {
            $category->user_id = $user->id;
        }

        if ($category->save()) {
            // event(new CategorySyncable($tag, $user, 'create'));
            return response()->json([
                'status' => 'success',
                'data' => Category::with('user')->find($category->id)
            ]);
        }
        $message = 'Category could not be added for some reason. Please try again later.';
        return response()->json([
            'status' => 'error',
            'message' => $message,
        ]);
    }

    public function update($id, Request $request)
    {
        $tag = Category::find($id);
        if ($tag->id) {
            $this->validate($request, [
                'name' => ["required", "string", "unique:categories,name,$id"],
                // 'department' => 'required'
            ]);
            $tag->name = $request->input('name');
            // $tag->department_id = $request->department;

        }

        if ($tag->update()) {
            // $message = 'Category updated successfully!';
            return response()->json([
                'status' => 'success',
                // 'message' => $message,
                'data' => Category::whereId($id)->with(['user'])->first()
            ]);
        }

        $message = 'Category was not updated. Please try again later.';
        return response()->json([
            'status' => 'error',
            'message' => $message
        ]);
    }

    public function delete($id)
    {
        $tag = Category::find($id);
        if ($tag->id && !$tag->items()->count()) {
            if ($tag->delete()) {
                // event(new CategorySyncable($tag, $request->user(), 'delete'));
                return response()->json(['status' => 'success', 'message' => "Category was successfully deleted!"]);
            } else {
                return response()->json([
                    'status' => 'error',
                    'message' => "Category could not be deleted! Please try again later."
                ]);
            }
        }
        return response()->json([
            'status' => 'error',
            'message' => "This category already has items attached to it."
        ]);

    }

    public function updateTrackedStatus($id)
    {
        $category = Category::find($id);
        $category->tracked = !$category->tracked;
        $category->update();

        return response()->json([
            'status' => 'success',
            'data' => $category->refresh()
        ]);
    }
}
