<?php

namespace App\Http\Controllers;

use App\Customer;
use App\Supplier;
use App\SupplierPayment;
use App\Supply;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CustomerController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    public function getCustomers()
    {
        return response()->json([
            'status' => 'success',
            'data' => Customer::whereState(true)
                ->with(['user', 'price_list'])
                ->latest()->get()
        ], 200);
    }


    public function createCustomer(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'phone' => 'required|min:9',
            'address' => 'required',
            'price_list' => 'nullable|numeric'
        ]);

        $customer = new Customer();
        $customer->name = $request->input('name');
        $customer->phone = $request->input('phone');
        $customer->address = $request->input('address');
        if (Schema::hasColumn('customers', 'code')) {
            $customer->code = get_next_ref('customer');
        }
        $customer->nic = $request->input('cni');
        $customer->email = $request->input('email');
        $customer->user_id = auth()->id();
        $customer->price_list_id = $request->input('price_list');

        $customer->save();
        return response()->json([
            'status' => 'success',
            'data' => $customer
        ], 200);
    }

    public function updateCustomer($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'phone' => 'required|size:9',
            'address' => 'required',
            'price_list' => 'nullable'
        ]);

        $customer = Customer::find($id);
        $customer->name = $request->input('name');
        $customer->address = $request->input('address');
        $customer->phone = $request->input('phone');
        $customer->nic = $request->input('cni');
        $customer->email = $request->input('email');
        // $customer->price_list_id = request()->input('price_list');

        $customer->update();
        return response()->json([
            'status' => 'success',
            'data' => $customer
        ], 200);
    }

    public function getArchiveCustomers(): JsonResponse
    {
        return response()->json([
            'status' => 'success',
            'data' => Customer::where('state', 0)->get(),
        ], 200);
    }

    public function updateCustomerPriceList($customer_id, $pl_id = null): JsonResponse
    {
        $customer = Customer::find($customer_id);
        $customer->price_list_id = $pl_id;
        $customer->save();

        return response()->json([
            'status' => 'success',
            'data' => $customer
        ]);
    }

    public function archiveCustomer($id): JsonResponse
    {
        $customer = Customer::find($id);
        $customer->state = !$customer->state;
        $customer->save();

        return response()->json([
            'status' => 'success',
            'state' => $customer->state
        ], 200);
    }

    public function updateCreditCustomer ($id): JsonResponse
    {

        $customer = Customer::find($id);
        $customer->buy_on_credit = !$customer->buy_on_credit;
        $customer->update();

        return response()->json([
            'status' => 'success',
            'data' => $customer
        ]);
    }

}
