<?php

namespace App\Http\Controllers;

use App\AccountSubcategory;
use App\Debt;
use App\DebtPayment;
use App\DebtType;
use App\Employee;
use App\EmployeePayment;
use Illuminate\Http\Request;

class DebtController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function getDebts()
    {
        return response()->json([
            'status' => 'success',
            'data' => Debt::where('state', 1)->with(['user', 'type', 'employee'])->latest()->get()
        ], 200);
    }

    public function getEmployeeDebts($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => Debt::whereEmployeeId($id)->where('state', 1)->with(['user', 'type', 'employee'])->latest()->get()
        ], 200);
    }

    public function getEmployeeArchivedDebts($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => Debt::whereEmployeeId($id)->where('state', false)->with(['user', 'type'])->latest()->get()
        ], 200);
    }

    public function getDebt($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => Debt::with(['type', 'user'])->find($id)
        ], 200);
    }

    public function createDebt(Request $request)
    {
        $this->validate($request, [
            'employee_id' => 'required',
            'type_id' => 'required',
            'amount' => 'required',
            'description' => 'required',
            'date' => 'required',
        ], [
            'employee_id.required' => 'Please Provide a employee_id',
            'amount.required' => 'Please Provide amount number',
            'description.required' => 'Please Provide a description',
            'date.required' => 'Please Provide a date',
            'type_id.required' => 'Please Provide a debt type',
        ]);
        $amount = $request->input('amount');
        $description = $request->input('description');

        $debt_type = DebtType::find($request->input('type_id'));
        $employee = Employee::find($request->input('employee_id'));
        $debt = new Debt();

        $debt->employee_id = $request->input('employee_id');
        $debt->amount = $amount;
        $debt->balance = $debt_type->can_pay ? $amount : 0;
        $debt->type_id = $request->input('type_id');
        $debt->description = $description;
        $debt->date = $request->input('date');
        $debt->user_id = auth()->id();

        if ($debt_type->affects_account) {
            $account_sub_category = AccountSubcategory::where('slug', 'like', '%debt%')->first();
            if (!$account_sub_category) {
                return response()->json([
                    'type' => 'error',
                    'message' => "Please ensure that an account subcategory exists called `...debt...`.
                    This will be used to register the payment."
                ], 403);
            }
            $account_subcategory_id = $account_sub_category['id'];
            $withdraw = new WithdrawalController();
            $req = $request;
            $req->request->add([
                'message' => "Debt withdrawal by $employee->name",
                'subcategory_id' => $account_subcategory_id
            ]);
            $withdraw->makeWithdrawal($req);
        }

        $debt->save();
        if ($debt_type->can_pay) {
            $employee->increment('total_debt', $amount);
        }

        return response()->json([
            'status' => 'success',
            'data' => Debt::with(['type', 'user', 'employee'])->find($debt->id)
        ], 200);
    }

    public function payDebt($id, Request $request)
    {
        $this->validate($request, [
            'amount' => 'required|numeric',
            'date' => 'required|date',
            'message' => 'required|string'
        ]);

        $debt = Debt::find($id);
        $employee = Employee::find($debt->employee_id);
        # return $debt;
        if ($request->amount > $debt->balance) {
            return response()->json([
                'status' => 'error',
                'message' => "The payment amount (XAF $request->amount)
                cannot be more than the payable balance (XAF $debt->balance)"
            ], 403);
        }

        // if ($debt->type->affects_account) {
        $sub_category = AccountSubcategory::where('slug', 'like', "%debt%")->first();
        if (!$sub_category) {
            return response()->json([
                'type' => 'error',
                'message' => "Please ensure that an account subcategory exists called `...debt...`.
                    This will be used to register the payment."
            ], 403);
        }
        $depositCtrl = new DepositController();
        $req = $request;
        $req->request->add([
            // 'amount' => $request->amount,
            'message' => $request->message,
            // 'date' => $request->date,
            'subcategory_id' => $sub_category->id
        ]);
        $depositCtrl->makeDeposit($req);
        // Add payment history
        $debt_payment = new DebtPayment();
        $debt_payment->debt_id = $id;
        $debt_payment->debt_type_id = $debt->type->id;
        $debt_payment->initial = $debt->balance;
        $debt_payment->amount = $request->amount;
        $debt_payment->balance = $debt->balance - $request->amount;
        $debt_payment->message = $request->message;
        $debt_payment->date = $request->date;
        $debt_payment->user_id = auth()->id();
        $debt_payment->employee_id = $debt->employee_id;
        $debt_payment->save();

        if ($debt->type->can_pay) {
            $employee->decrement('total_debt', $request->amount);
            $debt->decrement('balance', $request->amount);
        }
        $emp_payment_ctrl = new EmployeePaymentController();
        $req = $request;
        $req->request->add([
            'type' => 'incoming',
            'employee_id' => $employee->id,
            'message' => $request->message,
            // 'amount' => $request->amount,
            // 'date' => $request->date
        ]);
        $emp_payment_ctrl->createEmployeePayment($req);
        $debt->refresh();
        if ($debt->balance <= 0) {
            $debt->state = false;
        }
        $debt->update();

        return response()->json([
            'status' => 'success',
            'data' => $debt
        ]);
    }

    public function changeState($id)
    {
        $debt = Debt::find($id);
        $debt->state = !$debt->state;
        $debt->save();

        return response()->json([
            'status' => 'success',
            'debt' => $debt
        ], 200);
    }

    public function getArchiveDebts()
    {
        return response()->json([
            'status' => 'success',
            'data' => Debt::where('state', 0)->get(),
        ], 200);
    }

    public function getAllPayments()
    {
        return response()->json([
            'status' => 'success',
            'data' => EmployeePayment::whereType('incoming')->with('user')->latest()->get()
        ]);
    }

    public function getPaymentHistory($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => DebtPayment::whereDebtId($id)->with('user')->get()
        ]);
    }

    public function deleteDebt($id)
    {
        $debt = Debt::find($id);
        if (!$debt) {
            return response()->json([
                'status' => 'error',
                'message' => 'No matching debt found!'
            ], 404);
        }
        if ($debt->state != 0) {
            return response()->json([
                'status' => 'error',
                'message' => 'Cannot delete unarchived debt'
            ], 403);
        }
        if ($debt->balance > 0) {
            return response()->json([
                'status' => 'error',
                'message' => 'Cannot delete unpaid debt. Please make payment'
            ], 403);
        }
        $debt->delete();
        return response()->json([
            'status' => 'success',
            'data' => $debt
        ], 200);

    }

    public function getTrash()
    {
        return response()->json([
            'status' => 'success',
            'data' => Debt::onlyTrashed()->orderBy('deleted_at')->get()
        ]);
    }

    public function restoreTrash($id)
    {
        $debt = Debt::onlyTrashed()->find($id);
        $debt->restore();
        $debt->refresh();
        return response()->json([
            'status' => 'success',
            'data' => $debt
        ]);
    }

}
