<?php

namespace App\Http\Controllers;

use App\DebtType;
use App\Deposit;
use App\Account;
use App\AccountCategory;
use App\AccountSubcategory;
use App\EmployeeType;
use App\Item;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class DebtTypeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function getTypes()
    {
        return response()->json([
            'status' => 'success',
            'data' => DebtType::with('user')->whereState(1)->latest()->get()
        ], 200);
    }

    public function getAllTypes($all = true)
    {
        return response()->json([
            'status' => 'success',
            'data' => DebtType::with('user')->latest()->get()
        ], 200);
    }

    public function getType($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => DebtType::find($id)
        ], 200);
    }

    public function createType(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'can_pay' => 'required',
            'affects_account' => 'required',
        ], [
            'name.required' => 'Please Provide a name',
            'can_pay.required' => 'Please specify if the dept type can be paid back',
            'affects_account.required' => 'Please state if debt can affect main account',
        ]);
        $debt_type = new DebtType();
        $debt_type->name = $request->input('name');
        $debt_type->can_pay = $request->input('can_pay');
        $debt_type->affects_account = $request->input('affects_account');
        $debt_type->user_id = Auth::id();
        $debt_type->save();

        return response()->json([
            'status' => 'success',
            'data' => $debt_type
        ], 200);
    }

    public function updateDebtType($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'can_pay' => 'required',
            'affects_account' => 'required',
        ], [
            'name.required' => 'Please Provide a name',
            'can_pay.required' => 'Please specify if the dept type can be paid back',
            'affects_account.required' => 'Please state if debt can affect main account',
        ]);

        $debt_type = DebtType::find($id);
        if ($debt_type->debts()->count()) {
            return response()->json([
                'type' => 'error',
                'message' => "This category is readonly because one or more debts are of this type."
            ], 400);
        }
        $debt_type->name = $request->input('name');
        $debt_type->can_pay = $request->input('can_pay');
        $debt_type->affects_account = $request->input('affects_account');
        $debt_type->update();

        return response()->json([
            'status' => 'success',
            'data' => $debt_type
        ], 200);
    }

    public function archiveDebtType($id)
    {
        $debt = DebtType::find($id);
        $debt->state = !$debt->state;
        $debt->save();

        return response()->json([
            'status' => 'success',
            'data' => $debt
        ], 200);
    }
}
