<?php

namespace App\Http\Controllers;

use App\Department;
use App\Item;
use App\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;

class DepartmentController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function getDepartments()
    {
        return response()->json([
            'status' => 200,
            'data' => Department::whereState(1)->with('user')->latest()->get(),
        ]);
    }

    public function getMyDepartments()
    {
        return response()->json([
            'status' => 200,
            'data' => auth()->user()->department,
        ]);
    }

    public function getDepartmentsWithWarehouseItem($item_id = null): JsonResponse
    {
        $query = Department::whereState(1);
        if ($item_id) {
            $data = $query->with(['warehouse.items' => function ($wh) use ($item_id) {
                $wh->where('item_id', $item_id);
            }])->latest()->get();
        } else {
            $data = $query->with(['warehouse'])->latest()->get();
        }
        return response()->json([
            'status' => 200,
            'data' => $data
        ]);
    }

    public function getAllDepartments()
    {
        return response()->json([
            'status' => 200,
            'data' => Department::with('user')->latest()->get(),
        ]);
    }

    public function getDepartmentsWithWarehouse()
    {
        return response()->json([
            'status' => 200,
            'data' => Department::whereHas('warehouse_id')->latest()->get(),
        ]);
    }

    public function createDepartment(Request $request)
    {
        $this->validate($request, [
            'name' => 'required'
        ]);
        $department_name = $request->input('name');

        $count = Department::whereName($department_name)->count();
        if ($count) {
            $num = $count;
            $free = false;
            while (!$free) {
                $num++;
                $free = !Department::whereName($department_name . " $num")->exists();
            }
            $department_name = $department_name . ' ' . $num;
        }

        $department = new Department();
        $department->name = $department_name;
        $department->user_id = auth()->id();
        $department->warehouse_id = null;

        $department->save();
        return response()->json([
            'status' => 'success',
            'data' => Department::with('user')->latest()->first()
        ]);
    }

    public function updateDepartment($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required'
        ], [
            'name.required' => 'Please provide a department name'
        ]);

        $department = Department::find($id);
        $department_name = $request->input('name');
        if ($department_name != $department->name) {
            $count = Department::whereName($department_name)->count();
            if ($count) {
                $num = $count;
                $free = false;
                while (!$free) {
                    $num++;
                    $free = !Department::where([['name', $department_name . " $num"], ['id', '!=', $id]])->exists();
                }
                $department_name = $department_name . ' ' . $num;
            }

            $department->name = $department_name;
            $department->update();
        }

        return response()->json([
            'status' => 'success',
            'data' => Department::with('user')->find($id)
        ]);
    }

    public function setWarehouse($id, $wh)
    {
        $department = Department::find($id);
        $department->warehouse_id = $wh;
        $department->update();

        return response()->json([
            'status' => 'success',
            'data' => Department::with('user')->find($id)
        ]);
    }

    public function getArchivedDepartment()
    {
        return response()->json([
            'status' => 'success',
            'data' => Department::whereState(false)->with('user')->get()
        ]);
    }

    public function toggleState($id): JsonResponse
    {
        $department = Department::find($id);
        $department->state = !$department->state;
        $department->update();

        return response()->json([
            'status' => 'success',
            'data' => Department::with('user')->find($id)
        ]);
    }

    public function updateDepartmentSettings($id, Request $request)
    {
        $department = Department::find($id);
        $department->update(['settings' => $request->all()]);

        return response()->json([
            'status' => 'success',
            'data' => $department
        ]);
    }

}
