<?php

namespace App\Http\Controllers;

use App\Debt;
use App\DebtType;
use App\Employee;
use App\EmployeePayment;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Schema;

class EmployeeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function getEmployees()
    {
        return response()->json([
            'status' => 'success',
            'data' => Employee::where('state', 1)->latest()->with(['user', 'type'])->get()
        ], 200);
    }

    public function getEmployee($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => Employee::with('type')->find($id)
        ], 200);
    }

    /*
     * @todo Implement Edit Employee Function
     **/
    public function createEmployee(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'phone' => 'nullable',
            'nic' => 'nullable',
            'address' => 'nullable',
            'type_id' => 'required'
        ], [
            'name.required' => 'Please Provide a name',
            'phone.required' => 'Please Provide phone number',
            'nic.required' => 'Please Provide ID Number',
            'address.required' => 'Please Provide an address',
            'type_id.required' => 'Please Provide an employee type',
        ]);
        $employee = new Employee();
        $employee->name = $request->input('name');
        $employee->phone = $request->input('phone');
        $employee->email = $request->input('email');
        $employee->nic = $request->input('nic');
        $employee->address = $request->input('address');
        $employee->employment_date = $request->input('employment_date');
        $employee->type_id = $request->input('type_id');
        $employee->user_id = Auth::id() || $request->input('user_id');

        $employee->bound_user_id = $request->input('user_id');
        // if (Schema::hasColumn('employees', 'bound_user_id') && $request->input('user_id', false)) {}

        $employee->save();

        return response()->json([
            'status' => 'success',
            'employee' => $employee
        ], 200);
    }

    public function changeState($id)
    {
        $employee = Employee::find($id);
        $employee->state = !$employee->state;
        $employee->save();

        return response()->json([
            'status' => 'success',
            'employee' => $employee->name,
            'state' => $employee->state
        ], 200);
    }

    public function getArchiveEmployees()
    {
        return response()->json([
            'status' => 'success',
            'data' => Employee::where('state', 0)->with(['type', 'user'])->latest()->get(),
        ], 200);
    }

    public function getEmployeePayments($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => EmployeePayment::where([['employee_id', $id]])->with(['user'])->latest()->get(),
        ], 200);
    }


    public function createUnattendedDebt($id, Request $request)
    {
        $this->validate($request, [
            'employee_id' => 'required',
            'type_id' => 'required',
            'amount' => 'required',
            'description' => 'required',
            'date' => 'required',
        ], [
            'employee_id.required' => 'Please Provide a employee_id',
            'amount.required' => 'Please Provide amount number',
            'description.required' => 'Please Provide a description',
            'date.required' => 'Please Provide a date',
            'type_id.required' => 'Please Provide a debt type',
        ]);

        $amount = $request->input('amount');
        $description = $request->input('description');

        $debt_type = DebtType::find($request->input('type_id'));
        $debt = new Debt();

        $debt->employee_id = $request->input('employee_id');
        $debt->amount = $amount;
        $debt->balance = $debt_type->can_pay ? $amount : 0;
        $debt->type_id = $request->input('type_id');
        $debt->description = $description;
        $debt->date = $request->input('date');
        $debt->user_id = auth()->id();
        $debt->save();

        return response()->json([
            'status' => 'success',
            'data' => Debt::with(['type', 'user', 'employee'])->find($debt->id)
        ], 200);
    }

    public function checkBoundUser($user_id)
    {
        return response()->json([
            'status' => 'success',
            'bound' => Employee::whereBoundUserId($user_id)->exists()
        ]);
    }
}
