<?php

namespace App\Http\Controllers;

use App\Config;
use App\Packages\JsonStorage\JsonStorage;
use Carbon\Carbon;
use Carbon\Exceptions\InvalidFormatException;
use Illuminate\Contracts\Encryption\DecryptException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;

class LicenseController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    public function setup_license(Request $request)
    {
        $store = new JsonStorage();
        $data = $request->all();

        $server_date = $data['server_date'];
        $local_addr = $request->server('REMOTE_ADDR') == "127.0.0.1";
        $now = date('Y-m-d H:i:s');
        $local_server = $request->server('SERVER_NAME') == "localhost";

        // Compare date on local server
        if (Carbon::parse($server_date)->diffInHours($now) >= 23 && ($local_addr || $local_server)) {
            return response()->json([
                'message' => "Computer date doesn't seem to be correct. Please, try correcting the date & try again."
            ], 502);
        }

        $store->set('license', [
            'start_date' => $data['start_date'],
            'expires' => Crypt::encryptString($data['end_date']),
            'end_date' => $data['end_date'],
            'fetched' => true,
            'last_sync' => $data['server_date'],
            'tampered' => false,
            'last_verify' => $now,
            'client' => $data['client']['name']
        ]);

        // Set organisation's name from license server.
        // @todo Set other fields from license like address & contact.
        // @todo Make it possible to manually refresh license from client side.
        $config = Config::latest()->first();
        if (!$config) {
            $config = new Config();
            $config->save();
        }
        $config->company_name = $data['client']['name'];
        $config->update();

        $store->save();
        return ['status' => 'success', 'data' => $store->get('license', [])];
    }

    public function verify_license()
    {
        $store = new JsonStorage();
        $lc = $store->get('license', false);
        $active = false;
        $expires = null;
        $expires_in = null;
        $last_sync = null;
        $today = Carbon::now();
        $client = null;
        $end_date = null;
        if ($lc) {
            if (isset($lc['expires'])) {
                try {
                    $decrypted = Crypt::decryptString($lc['expires']);
                    $expiration = Carbon::parse($decrypted);
                    $active = $expiration->isAfter($today);
                    $expires = $expiration->format('Y-m-d');
                    $expires_in = $today->diffInDays($expiration, false);
                    $last_sync = $lc['last_sync'] ?? false;
                    $client = $lc['client'];
                    $end_date = $lc['end_date'];
                } catch (DecryptException | InvalidFormatException $e) {
                    // $license probably compromised.
                    // @todo Notify client to prefetch
                    return response()->json([
                        'message' => "Invalid license. Please, fetch your license again."
                    ], 502);
                }
            }
        }
        if (!$active) {
            return response()->json([
                'message' => ("You license has expired (since: " . $expires . "). Please, renew your license to continue using " . env('APP_NAME'))
            ], 502);
        }
        return [
            'status' => 'success',
            'active' => true,
            'expiration_date' => $expires,
            'expires_in' => $expires_in,
            'end_date' => $end_date,
            'last_sync' => $last_sync,
            'now' => Carbon::now(),
            'client' => $client
        ];
    }
}
