<?php

namespace App\Http\Controllers;

use App\Modifier;
use App\ModifierOption;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;

class ModifierController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }


    public function index(): JsonResponse
    {
        $modifiers = Modifier::with('user')->get();
        return response()->json([
            'status' => 'success',
            'data' => $modifiers
        ]);
    }

    /**
     * @throws ValidationException
     */
    public function store(Request $request): JsonResponse
    {

        $this->validate($request, [
            'name' => 'required',
            'options' => 'required|array'
        ]);

        $modifier = new Modifier();
        $modifier->name = $request->input('name');
        $modifier->user_id = auth()->id();
        $modifier->save();

        $modifier->options()->saveMany(
            array_map(function ($option) {
                return new ModifierOption([
                    'name' => $option['name'],
                    'price' => $option['price'],
                    'user_id' => auth()->id()
                ]);
            }, $request->options)
        );

        return response()->json([
            'status' => 'success',
            'data' => $modifier->load('user')->refresh()
        ]);
    }

    public function destroy($id)
    {
        $modifier = Modifier::find($id);
        $modifier->delete();
        return response()->json([
            'status' => 'success',
            'data' => $modifier
        ], 200);
    }

    public function getOptions()
    {
        $modifier_options = ModifierOption::all();
        return response()->json([
            'status' => 'success',
            'data' => $modifier_options
        ]);
    }

    public function getModifierOptions($modifier_id)
    {
        $options = ModifierOption::all();
        $modifier_options = [];
        foreach ($options as $option) {
            if ($option->modifier_id == $modifier_id) {
                $modifier_options[] = $option;
            }
        }
        return response()->json([
            'status' => 'success',
            'data' => $modifier_options
        ]);
    }

    public function storeOption(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'modifier_id' => 'required|numeric'
        ]);

        $modifier_option = new ModifierOption();
        $modifier_option->name = $request->input('name');
        $modifier_option->price = $request->input('price');
        $modifier_option->modifier_id = $request->input('modifier_id');
        $modifier_option->user_id = auth()->id();
        $modifier_option->save();

        return response()->json([
            'status' => 'success',
            'data' => $modifier_option->refresh()
        ]);
    }

    public function updateOption($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'modifier_id' => 'required|numeric'
        ]);
        $modifier_option = ModifierOption::where('id', $id)->first();
        $modifier_option->update($request->all());

        return response()->json([
            'status' => 'success',
            'data' => $modifier_option->refresh()
        ]);

    }

    public function update($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required'
        ]);
        $modifier = Modifier::where('id', $id)->first();
        $modifier->update($request->all());

        return response()->json([
            'status' => 'success',
            'data' => $modifier->refresh()
        ]);

    }

    public function destroyOption($id)
    {
        $modifier_option = ModifierOption::find($id);
        $modifier_option->delete();
        return response()->json([
            'status' => 'success',
            'data' => $modifier_option
        ], 200);
    }


}
