<?php

namespace App\Http\Controllers;

use App\POS;
use App\Store;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;

class POSController extends Controller
{

    public function index($id = null): JsonResponse
    {
        return response()->json([
            'status' => 'success',
            'data' => POS::with('store')->get()
        ]);
    }

    /**
     * @throws ValidationException
     */
    public function store(Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => 'required',
            'store' => 'required'
        ], [
            'store.required' => 'Please, specify a store'
        ]);

        $pos = new POS();
        $pos->name = $request->input('name');
        $pos->user_id = auth()->id();
        $pos->store_id = $request->input('store');
        $pos->code = get_next_ref('pos');
        $pos->active = $request->has('active') ? $request->input('active') : false;

        $pos->save();

        return response()->json([
            'status' => 'success',
            'data' => $pos->refresh()
        ]);
    }

    /**
     * @throws ValidationException
     */
    public function update(POS $pos, Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => 'required'
        ]);

        $pos->name = $request->input('name');

        $pos->update();

        return response()->json([
            'status' => 'success',
            'data' => $pos->refresh()
        ]);
    }

    public function change_status(POS $pos): JsonResponse
    {
        $pos->update(['active' => !$pos->active]);

        return response()->json($pos->refresh());
    }

    /**
     * @throws ValidationException|Exception
     */
    public function delete(POS $pos): JsonResponse
    {
        $pos->delete();

        return response()->json([
            'status' => 'success'
        ]);
    }
}
