<?php

namespace App\Http\Controllers;

use App\PaymentMethod;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class PaymentMethodController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function getPaymentMethods(): JsonResponse
    {
        return response()->json([
            'status' => 200,
            'data' => PaymentMethod::with('user')->get(),
        ]);
    }

    public function getActivePaymentMethods(): JsonResponse
    {
        return response()->json([
            'status' => 200,
            'data' => PaymentMethod::active()->orderBy('default', 'desc')->get(),
        ]);
    }

    public function store(Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => ['required', Rule::unique('payment_methods')]
        ]);
        $name = $request->input('name');

        $method = new PaymentMethod();
        $method->name = $name;
        $method->user_id = auth()->id();
        $method->status = true;
        $method->default = false;
        $method->save();

        if (!PaymentMethod::active()->count()) {
            // Set default if non has been set
            $this->setDefault($method->id);
        }

        return response()->json([
            'status' => 'success',
            'data' => PaymentMethod::with('user')->find($method->id)
        ]);
    }

    public function updatePaymentMethod($id, Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => ['required', Rule::unique('payment_methods')->ignore($id)]
        ]);

        $method = PaymentMethod::with('user')->find($id);

        $method->name = $request->input('name');
        $method->update();

        return response()->json([
            'status' => 'success',
            'data' => $method
        ]);
    }

    public function setDefault($id): JsonResponse
    {
        PaymentMethod::default()->decrement('default');
        PaymentMethod::find($id)->increment('default');

        return response()->json([
            'status' => 'success'
        ]);
    }

    public function toggleStatus($id): JsonResponse
    {
        $method = PaymentMethod::with('user')->find($id);
        $method->status = !$method->status;
        $method->update();

        return response()->json([
            'status' => 'success',
            'data' => $method
        ]);
    }

    public function delete ($id): JsonResponse
    {
        $method = PaymentMethod::find($id);
        if ($method->sales()->count()) {
            return response()->json([
                'status' => 'error',
                'message' => 'Payment method cannot be deleted because it is bound to a payment'
            ]);
        }
        $method->delete();
        return response()->json(['status' => 'success']);
    }

}
