<?php

namespace App\Http\Controllers;

use App\PriceList;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class PriceListController extends Controller
{

    public function getPriceLists(): JsonResponse
    {
        return response()->json([
            'status' => 'success',
            'data' => PriceList::orderBy('position')->get()
        ]);
    }

    public function addPriceList(Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => 'required',
            'position' => 'required'
        ]);

        $price_list = new PriceList();
        $price_list->name = $request->input('name');
        $price_list->description = $request->input('description');
        $price_list->position = ++PriceList::latest()->first()->position;
        $price_list->user_id = auth()->id();

        $price_list->save();
        return response()->json(['status' => 'success', 'data' => $price_list]);
    }

    /**
     * @throws \Exception
     */
    public function removePriceList($id)
    {
        $price_list = PriceList::find($id);
        if ($price_list->system) {
            return response()->json([
                'status' => 'error',
                'message' => 'This price list cannot be deleted.',
            ]);
        }
        if ($price_list->items()->count() or $price_list->customers()->count()) {
            return response()->json([
                'status' => 'warning',
                'message' => 'Price list cannot be removed due to item or customer attachment.',
            ]);
        }
        $price_list->delete();
        return response()->json(['status' => 'success']);
    }
}
