<?php

namespace App\Http\Controllers;

use App\Config;
use App\Debt;
use App\DebtType;
use App\Deposit;
use App\Employee;
use App\Exports\ExpectedSalariesExport;
use App\Item;
use App\Salary;
use App\Stock;
use App\Supplier;
use App\SupplierPayment;
use App\Supply;
use App\Warehouse;
use App\Withdrawal;
use FontLib\Table\Type\name;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Laravel\Lumen\Routing\Controller as BaseController;
use Maatwebsite\Excel\Facades\Excel;
use PDF;
use function Illuminate\Support\Facades\View;


class ReportController extends BaseController
{

    /**
     * @var string
     */
    /**
     * @var false|string
     */
    public $file_start, $path, $tmp_path, $info;

    /**
     * @var Config[]|\Illuminate\Database\Eloquent\Collection
     */

    public function __construct()
    {
        set_time_limit(1000);
        $this->path = storage_path('app/public/');
        $this->tmp_path = storage_path('app/public/tmp/');
        $this->file_start = (date('Y-m-d His'));
        $this->info = Config::find(1);
    }

    /**
     * Likely not in use. Refactor.
    */
    public function index()
    {
        $suppliers = Supplier::whereState(true)->get();
        $pdf = PDF::loadView('reports/supplier_list', compact('suppliers'));
        $pdf->save($this->path . $this->file_start . '_file233.pdf');
        return \view('reports/supplier_list', compact('suppliers'));
    }

    public function filterSalaries(Request $request)
    {
        $data = $request->all();
        $query = Salary::whereNotNull('id');
        if (isset($data['start_date'])) {
            $query = $query->whereDate('created_at', '>=', $data['start_date']);
        }
        if (isset($data['end_date'])) {
            $query = $query->whereDate('created_at', '<=', $data['end_date']);
        }
        $query = $query->get();
        return response()->json([
            'status' => 'success',
            'data' => $query
        ]);
    }

    public function printFilteredSalaries(Request $request)
    {
        $res = $this->filterSalaries($request);
        $pdf = PDF::loadView('reports', ['data' => $res['data']]);
        $path = storage_path('app/public/');
        $file = ('tmp/' . date('Y-m-d His') . '_salaries.pdf');
        $pdf->save($path . $file);
        $url = url('storage/' . $file);
        return response()->json([
            'url' => $url
        ]);
    }

    /*DONE!!!!*/
    public function getSupplierPayment(Request $request)
    {
        $data = $request->all();
        $payments = SupplierPayment::with('supplier');

        if (isset($data['supplier'])) {
            $payments = $payments->whereIn('supplier_id', $data['supplier']);
        }

        if (isset($data['start_date'])) {
            $payments = $payments->whereDate('created_at', '>=', $data['start_date']);
        }
        if (isset($data['end_date'])) {
            $payments = $payments->whereDate('created_at', '<=', $data['end_date']);
        }
        $payments = $payments->get();
        $url = '';
        if (isset($data['dl'])) {
            $pdf = PDF::loadView('reports/supplier_payment', ['payments' => $payments]);
            $path = storage_path('app/public/');
            $filename = 'tmp/supplier_payment_' . $this->file_start . '.pdf';
            $pdf->save($path . $filename);
            $url = url('storage/' . $filename);
            // return \view('supplier_payment', ['payments'=> $payments, 'info' => $this->info]);

        }
        return response()->json([
            'status' => 'success',
            'data' => $payments,
            'url' => $url
        ]);
    }

    /*DONE!!!!*/
    public function _expectedExpenditure(Request $request)
    {
        $data = $request->all();

        $expenditure = Employee::where('state', 1)
            ->with(['type'])
            ->withCount(['debts' => function ($debt) {
                return $debt->where([['state', 1], ['balance', '>', 0]])
                    ->select(DB::raw('sum(balance)'));
            }])
            ->get();

        $total_payment = 0;
        $total_debt = 0;
        $url = '';
        foreach ($expenditure as $emp) {
            $total_payment += $emp->type->salary;
            $total_debt += $emp->debts_count;
        }
        $expenditure->total_payment = $total_payment;
        $expenditure->total_debt = $total_debt;
        if (isset($data['dl'])) {
            $pdf = PDF::loadView('reports/expected_expenditure', ['expenditure' => $expenditure]);
            $filename = 'tmp/expected_expenditure_' . $this->file_start . '.pdf';
            $pdf->save($this->path . $filename);
            $url = url('storage/' . $filename);
        }

        return response()->json([
            'status' => 'success',
            'data' => $expenditure,
            'total_payment' => $total_payment,
            'total_debt' => $total_debt,
            'url' => $url
        ]);
    }

    public function expectedExpenditure(Request $request)
    {
        $data = $request->all();
        $types = $request->types ?: [];

        $expenditure = Employee::where('state', 1)
            ->with(['debts' => function ($debt) use ($types) {
                return $debt->where([['state', 1], ['balance', '>', 0]])->whereIn('type_id', $types);
            }])
            ->get();

        $labels = [];
        $results = [];

        $index = 0;
        // $types = DebtType::whereIn('id', $data['types'])->select('name', 'id')->get();

        foreach ($expenditure as $emp) {
            $label = ['S/N', trans('reports.label.name'), trans('reports.label.department'), trans('reports.label.salary')];
            $salary = $emp->type->salary;
            $sub = [++$index, $emp->name, $emp->type->name, ($salary)];
            $total_debt = 0;
            foreach ($types as $type) {
                $sum = 0;
                $label[] = DebtType::find($type)->name;
                foreach ($emp->debts as $debt) {
                    if ($debt->type_id == $type) {
                        $sum += $debt->balance;
                        $total_debt += $debt->balance;
                    }
                }
                $sub[] = $sum;
            }
            $label[] = trans('reports.label.total_debt');
            $label[] = trans('reports.label.balance');
            $sub[] = $total_debt;
            $sub[] = $salary - $total_debt;

            if ($index == 1) {
                $labels = $label;
            }
            $results[] = $sub;
        }
        array_splice($results, 0, 0, [$labels]);

        /* return $result;
         $expenditure->total_payment = $total_payment;
         $expenditure->total_debt = $total_debt;*/

        $url = '';
        if ($request->dl) {
            if (strtolower($request->dl) == 'pdf') {

                $pdf = PDF::loadView('reports.expected_payroll', compact('results'));
                $path = storage_path('app/public/');
                $filename = 'tmp/expected_expenditure_' . $this->file_start . '.pdf';
                $pdf->save($path . $filename);
                $url = url('storage/' . $filename);

            } elseif (strtolower($request->dl) == 'excel') {

                $path = 'expected_payroll_' . date('YmdHis') . '.xlsx';
                $url = url('storage/' . $path);
                Excel::store(new ExpectedSalariesExport($results), 'public/' . $path);
            }
        }

        return response()->json([
            'status' => 'success',
            'data' => $results,
            // 'total_payment' => $total_payment,
            // 'total_debt' => $total_debt,
            'url' => $url
        ]);
    }

    /*DONE!!!!*/
    public function employeeDebts(Request $request)
    {
        $data = $request->all();
        $types = $request->types;
        $start_date = $request->start_date;
        $end_date = $request->end_date;

        $employees = Employee::with(['debts' => function ($debts) use ($types, $start_date, $end_date) {
            if ($types and is_array($types)) {
                $debts->whereIn('type_id', $types);
            }
            if ($start_date) {
                $debts->whereDate('date', '>=', $start_date);
            }
            if ($end_date) {
                $debts->whereDate('date', '<=', $end_date);
            }
            $debts->where('state', 1)->with('type');

        }])->whereHas('debts', function ($debts) use ($types) {
            $debts->where('state', 1);
        }, '>=', 1);

        if ($request->has('employee') and is_array($request->employee)) {
            $employees = $employees->whereIn('id', $request->employee);
        }
        $temp = $employees->get();
        $employees = [];
        foreach ($temp as $index => $employee) {
            if ($employee->debts->count()) {
                array_push($employees, $employee);
            }
        }
        $url = '';
        if (isset($data['dl'])) {
            $pdf = PDF::loadView('reports/employee_debts', ['data' => $employees, 'start_date' => $start_date, 'end_date' => $end_date]); // 'info' => $this->info
            $filename = 'tmp/employee_debts_' . $this->file_start . '.pdf';
            $pdf->save($this->path . $filename);
            $url = url('storage/' . $filename);
        }

        return response()->json([
            'status' => 'success',
            'data' => $employees,
            'url' => $url
        ]);
    }

    /*DONE!!!!*/
    function getSupplies(Request $request)
    {
        $data = $request->all();
        $supplies = Supply::with(['supplier', 'user']);
        if (isset($data['supplier'])) {
            $supplies = $supplies->whereIn('supplier_id', $data['supplier']);
        }
        if ($data['items'] ?? false) {
            $supplies = $supplies->whereHas('items', function ($query) use($data) {
                $query->whereIn('item_id', $data['items']);
            });
        }
        if (isset($data['start_date'])) {
            $supplies = $supplies->whereDate('created_at', '>=', $data['start_date']);
        }
        if (isset($data['end_date'])) {
            $supplies = $supplies->whereDate('created_at', '<=', $data['end_date']);
        }
        $supplies = $supplies->with('items.item')->get();

        $url = '';
        if (isset($data['dl'])) {
            $pdf = PDF::loadView('reports/supplies', ['supplies' => $supplies]);
            $filename = 'tmp/supplies_' . $this->file_start . '.pdf';
            $pdf->save($this->path . $filename);
            $url = url('storage/' . $filename);
        }
//        return \view('reports/supplies', ['supplies'=> $supplies, 'info'=> $this->info]);
        return response()->json([
            'status' => 'success',
            'data' => $supplies,
            'url' => $url
        ]);
    }

    public function deposits(Request $request)
    {
        $data = $request->all();
        $url = '';

        $deposits = Deposit::with(['user', 'subcategory']);
        if (isset($data['account_subcategory_id'])) {
            $deposits = $deposits->whereIn('account_subcategory_id', $data['account_subcategory_id']);
        }
        if (isset($data['start_date'])) {
            $deposits = $deposits->whereDate('date', '>=', $data['start_date']);
        }
        if (isset($data['end_date'])) {
            $deposits = $deposits->whereDate('date', '<=', $data['end_date']);
        }
        $deposits = $deposits->select(DB::raw('account_subcategory_id, user_id, amount, message, user_id, created_at, id'))
            ->groupBy('account_subcategory_id', 'user_id', 'amount', 'message', 'created_at', 'id')->get();

        if (isset($data['dl'])) {
            $pdf = PDF::loadView('reports/deposits', ['deposits' => $deposits]);
            $path = storage_path('app/public/');
            $filename = 'tmp/deposits_' . $this->file_start . '.pdf';
            $pdf->save($path . $filename);
            $url = url('storage/' . $filename);
            // return \view('deposits', compact('deposits'));
        }
        return response()->json([
            'status' => 'success',
            'data' => $deposits,
            'url' => $url
        ]);
    }

    public function withdrawals(Request $request)
    {
        $data = $request->all();
        $url = '';

        $withdrawals = Withdrawal::with('subcategory');
        if (isset($data['account_subcategory_id'])) {
            $withdrawals = $withdrawals->whereIn('account_subcategory_id', $data['account_subcategory_id']);
        }
        if (isset($data['start_date'])) {
            $withdrawals = $withdrawals->whereDate('date', '>=', $data['start_date']);
        }
        if (isset($data['end_date'])) {
            $withdrawals = $withdrawals->whereDate('date', '<=', $data['end_date']);
        }
        $withdrawals = $withdrawals->select(DB::raw('account_subcategory_id, user_id, amount, message, user_id, created_at, id'))
            ->groupBy('account_subcategory_id', 'user_id', 'amount', 'message', 'created_at', 'id')->get();


        if (isset($data['dl'])) {
            $pdf = PDF::loadView('reports/withdrawals', ['withdrawals' => $withdrawals, 'info' => $this->info]);
            $filename = 'tmp/withdrawals_' . $this->file_start . '.pdf';
            $pdf->save($this->path . $filename);
            $url = url('storage/' . $filename);
            // return \view('withdrawals', compact('withdrawals'));
        }
        return response()->json([
            'status' => 'success',
            'data' => $withdrawals,
            'url' => $url
        ]);
    }

    /*DONE!!!!*/
    function getInventory(Request $request)
    {

        $items = Item::select(['id', 'name', 'unit_price', 'purchase_price', 'category_id'])
            ->with(['category'])
            ->where('item_type', 'product')->get();
        $inventories = [];
        $data_with_empty = [];
        $data_all = [];

        foreach ($items as $item) {
            $qty = Stock::query();
            if ($request->filled('warehouses') && count($request->warehouses)) {
                $qty = $qty->whereIn('warehouse_id', $request->input('warehouses'))->where('state', 1);
            }
            $qty = $qty->where('item_id', $item->id)->sum('quantity');

            if ($qty > 0) {
                if (!isset($data_all[$item->category->name])) {
                    $data_all[$item->category->name] = [];
                }
                $data_all[$item->category->name][] = ['name' => $item['name'], 'quantity' => $qty, 'selling_price' => $item['unit_price'], 'purchase_price' => $item['purchase_price']];
            }
            if ($qty >= 0) {
                if (!isset($data_with_empty[$item->category->name])) {
                    $data_with_empty[$item->category->name] = [];
                }
                $data_with_empty[$item->category->name][] = ['name' => $item['name'], 'quantity' => $qty, 'selling_price' => $item['unit_price'], 'purchase_price' => $item['purchase_price']];
            }
        }
        if ($request->input('zero_items')) {
            $inventories = $data_with_empty;
        } else {
            $inventories = $data_all;
        }

        $expected_sales = (bool)$request->input('expected_sales') == true;
        $wh = Warehouse::find($request->input('warehouse'));
        $url = '';
        if ($request->input('dl')) {
            $pdf = PDF::loadView('reports/inventory', ['inventories' => $inventories, 'expected_sales' => $expected_sales, 'wh' => $wh]);
            $filename = 'tmp/inventories_' . $this->file_start . '.pdf';
            $pdf->save($this->path . $filename);
            $url = url('storage/' . $filename);
        }
//        return \view('reports/supplies', ['supplies'=> $supplies, 'info'=> $this->info]);
        return response()->json([
            'status' => 'success',
            'data' => $inventories,
            'url' => $url
        ]);
    }

}
