<?php

namespace App\Http\Controllers;

use App\AccountSubcategory;
use App\Debt;
use App\Employee;
use App\EmployeePayment;
use App\Salary;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Exports\ExpectedSalariesExport;
use Maatwebsite\Excel\Facades\Excel;

class SalaryController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function export()
    {
        return Excel::download(new ExpectedSalariesExport, 'users.xlsx');
    }

    public function getSalaries()
    {
        return response()->json([
            'status' => 'success',
            'data' => Salary::with(['user', 'employee'])->latest()->get()
        ], 200);
    }

    public function getSalary($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => Salary::find($id)
        ], 200);
    }

    public function getEmployeeSalary($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => EmployeePayment::where('employee_id', $id)->with('user')->latest()->get()
        ], 200);
    }

    public function createEmployeeSalary(Request $request)
    {
        $this->validate($request, [
            'employee_id' => 'required',
            'amount' => 'required',
            'pay_debts' => 'required|boolean'
        ], [
            'employee_id.required' => 'Please select an employee',
            'amount.required' => 'Please provide an amount'
        ]);

        $employee = Employee::find($request->employee_id);
        $salary_amt = $employee->type->salary;
        $payable_amt = $request->input('amount');
        $emp_debts_sum = 0;

        $date = $request->date;
        $paid = Salary::where([['employee_id', $employee->id]])
            ->whereMonth('date', '=', Carbon::parse($date)->format('m'))->first();
        if ($paid) {
            return response()->json([
                'status' => 'error',
                'message' => ("Salary for the month of <i>" . Carbon::parse($date)->format('F')
                    . "</i> was already issue.")
            ], 403);
        }

        $_debts = Debt::whereHas('type', function ($query) {
            return $query->where('can_pay', true);
        })->where([
            ['employee_id', $request->employee_id],
            ['state', true]
        ]);

        if ($request->input('pay_debts')) {
            $debt_sum = $_debts->sum('balance');
            if ($debt_sum > $payable_amt) {
                return response()->json([
                    'status' => 'error',
                    'message' => "The employee salary (XAF $payable_amt)
                    is less than the total debt (XAF $debt_sum).
                    Please, manually pay in the employee debts instead."
                ], 403);
            }
            $emp_debts_sum = $debt_sum;
            $debt_affect_acc = Debt::whereHas('type', function ($query) {
                return $query->where('affects_account', true);
            })->where([
                ['employee_id', $employee->id],
                ['state', true]
            ]);
            if ($debt_affect_acc->count()) {
                $debt_affect_acc_sum = $debt_affect_acc->sum('balance');
                $sub_category = AccountSubcategory::where('slug', 'like', "%debt%")->first();
                if (!$sub_category) {
                    return response()->json([
                        'type' => 'error',
                        'message' => "Please ensure that an account subcategory exists called `...debt...`.
                    This will be used to register this payment."
                    ], 403);
                }
                $dept_affect_acc_ids = $debt_affect_acc->pluck('balance', 'id')->all();
                foreach ($dept_affect_acc_ids as $dept_affect_acc_id => $debt_balance) {
                    $debtCtrl = new DebtController();
                    $subReq = $request;
                    $subReq->request->add([
                        'amount' => $debt_balance,
                        'date' => date("Y-m-d H:i:s"),
                        'message' => "Salary deduction"
                    ]);
                    $debtCtrl->payDebt($dept_affect_acc_id, $subReq);
                }
                /*$depositCtrl = new DepositController();
                $req = $request;
                $req->request->add([
                    'amount' => $debt_affect_acc_sum,
                    'message' => "Debt payment by $employee->name",
                    'subcategory_id' => $sub_category->id
                ]);
                $depositCtrl->makeDeposit($req);*/
            }
            // archive debts
            $_debts->update([
                'state' => false,
                'balance' => 0
            ]);
            $employee->total_debt = 0;
            $employee->update();
        }
        $emp_payment_ctrl = new EmployeePaymentController();
        $req = $request;
        $req->request->add([
            'type' => 'outgoing',
            'amount' => $payable_amt - $emp_debts_sum,
            'message' => "Salary payment to $employee->name"
        ]);
        $emp_payment_ctrl->createEmployeePayment($req);

        $salary = new Salary();
        $salary->user_id = auth()->id();
        $salary->employee_id = $employee->id;
        $salary->total_salary = $employee->type->salary; // $salary_amt;
        $salary->debt = $emp_debts_sum;
        $salary->paid = $payable_amt - $emp_debts_sum;
        $salary->date = $request->date;

        $salary->save();

        return response()->json(
            [
                'status' => 'success',
                'data' => $salary
            ]
        );
    }
}
