<?php

namespace App\Http\Controllers;

use App\Config;
use App\Sale;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use PDF;

class SaleController extends Controller
{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    public function getSale($id)
    {
        $sale = Sale::with(['user.department', 'items.item', 'cashier', 'service', 'customer', 'payments'])->find($id);
        $company = Config::latest()->first();
        return response()->json([
            'status' => 'success',
            'data' => ["sale_order" => $sale, "company_info" => $company]
        ], 200);
    }

    /**
     * @throws \Illuminate\Validation\ValidationException
     */
    public function printSales (Request $request): JsonResponse
    {
        $this->validate($request, [
            'sales' => 'required|array|min:1'
        ]);
        $sales = Sale::with(['items.item', 'cashier', 'service', 'customer'])
            ->whereIn('id', $request->input('sales', []))->get();

        $pdf = PDF::loadView('reports/invoice', ['sales' => $sales]);
        $path = storage_path('app/public/');
        $filename = 'tmp/sales_invoice_' . date('Y-m-d His') . '.pdf';
        $pdf->save($path . $filename);
        $url = url('storage/' . $filename);

        return response()->json([
            'status' => 200,
            'url' => $url
        ]);

    }

    public function index(Request $request): JsonResponse
    {
        $per_page = $request->input('per_page', 100);
        $sales = Sale::with(['items.item', 'cashier', 'service', 'customer', 'payments'])->withCount('items');
        $items = $request->input('items');
        $categories = $request->input('categories');
        $customers = $request->input('customers');
        $session = $request->input('session');
        $cashiers = $request->input('cashiers');
        $has_debt = $request->input('has_debt');
        $has_discount = $request->input('has_discount');
        $start_date = $request->input('start_date');
        $end_date = $request->input('end_date');

        $query = "";
        if ($session) {
            $sales  = $sales->where('session_id', $session);
            $query .= trans("sale.report.query.session");
        }
        if ($categories && count($categories)) {
            // The long way
            /*$category_items = Category::with(['items' => function ($q) {
                $q->select(['id', 'category_id']);
            }])->whereIn('id', $categories)->get();
            $item_ids = array_merge(...$category_items->map(function ($row) {
                return $row->items->map(function($item) {return $item['id']; })->toArray();
            }));
            $sales  = $sales->whereHas('items', function ($q) use($item_ids) {
                $q->whereIn('item_id', $item_ids);
            });*/
            // The short way
            $sales  = $sales->whereHas('items', function ($q) use($categories) {
                $q->whereHas('item', function ($q2) use ($categories) {
                    $q2->whereIn('category_id', $categories);
                });
            });
            $query .= trans("sale.report.query.category");
        }

        if ($items && count($items)) {
            $sales  = $sales->whereHas('items', function ($q) use($items) {
                $q->whereIn('item_id', $items)->with('item');
            });
            $query .= trans("sale.report.query.item");
        }
        if ($start_date) {
            $sales  = $sales->whereDate('created_at', '>=', $start_date);
            $query .= trans("sale.report.query.start_date");
        }
        if ($end_date) {
            $sales  = $sales->whereDate('created_at', '<=', $end_date);
            $query .= trans("sale.report.query.end_date");
        }
        if ($customers && count($customers)) {
            $sales  = $sales->whereIn('customer_id', $customers);
            $query .= trans("sale.report.query.customer");
        }
        if ($cashiers && count($cashiers)) {
            $sales  = $sales->whereIn('user_id', $cashiers);
            $query .= trans("sale.report.query.cashier");
        }
        if ($has_debt) {
            $sales  = $sales->whereRaw('debt > 0');
            $query .= trans("sale.report.query.debt");
        }
        if ($has_discount) {
            $sales  = $sales->whereRaw('discount > 0');
            $query .= trans("sale.report.query.discount");
        }
        $sales = $sales->latest();
        $url = '';
        if ($request->input('print')) {
            $pdf = PDF::loadView('reports/sales', ['sales' => $sales->get(), 'query' => $query]);
            $path = storage_path('app/public/');
            $filename = 'tmp/sales_' . date('Y-m-d His') . '.pdf';
            $pdf->save($path . $filename);
            $url = url('storage/' . $filename);
        }

        return response()->json([
            'status' => 200,
            'data' => $sales->paginate($per_page),
            'url' => $url
        ]);

    }

}
