<?php

namespace App\Http\Controllers;

use App\SalesReceipt;
use Auth;
use App\User;
use App\Source;
use Illuminate\Http\Request;
use App\Events\SourceSyncable;
use Illuminate\Support\Facades\Schema;
use Illuminate\Validation\Rule;

class SalesReceiptController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    public function index()
    {
        return response()->json([
            'status' => 'success',
            'data' => SalesReceipt::all()
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'title' => ['required', 'string']
        ]);
        $receipt = new SalesReceipt();
        $receipt->user_id = auth()->id();
        $receipt->title = $request->input('title');
        if (Schema::hasColumn('sales_receipts', 'system')) {
            $receipt->system = false;
        }

        $receipt->save();
        return response()->json([
            'status' => 'success'
        ]);
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param int $token
     * @return \Illuminate\Http\JsonResponse
     */
    public function changeStatus($id)
    {
        $receipt = SalesReceipt::find($id);
        if ($receipt) {
            $receipt->active = !$receipt->active;
            $receipt->update();
        }
        return response()->json([
            'status' => 'success',
            'data' => $receipt
        ]);
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param int $token
     * @return \Illuminate\Http\JsonResponse
     */
    public function delete($id)
    {
        $receipt = SalesReceipt::find($id);
        if ($receipt->system) {
            return response()->json([
                'status' => 'error'
            ], 405);
        }
        $receipt->delete();
        return response()->json([
            'status' => 'success'
        ]);
    }

}
