<?php

namespace App\Http\Controllers;

use App\Department;
use App\Role;
use App\Sale;
use App\Session;
use App\User;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;
use Mike42\Escpos\PrintConnectors\WindowsPrintConnector;
use Mike42\Escpos\Printer;

class ServiceController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    /**
     * @return JsonResponse
     */
    public function index()
    {
        $services = User::services()->with('user', 'department')->latest()->get();
        return response()->json([
            'status' => 'success',
            'data' => $services
        ]);
    }

    public function serviceProspects()
    {
        $services = User::nonServices()->active()->latest()->get();
        return response()->json([
            'status' => 'success',
            'data' => $services
        ]);
    }

    /**
     * @return JsonResponse
     */
    public function getDepartmentServicesFirst(): JsonResponse
    {
        // $services = User::services()->active()->where('department_id', auth()->user()->department_id)->latest()->get();
        $department_id = auth()->user()->department_id;
        $services = User::services()->active()->orderByRaw("department_id = $department_id DESC")->get();

        return response()->json([
            'status' => 'success',
            'data' => $services
        ]);
    }

    public function list()
    {
        $services = User::services()->active()->with(['department', 'user'])->latest()->get();
        return response()->json([
            'status' => 'success',
            'data' => $services
        ]);
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     * @throws ValidationException
     */
    public function store($user_id)
    {
        $role = Role::whereSlug('service')->first();
        $user = User::with('user', 'department')->find($user_id);
        if ($role) {
            $user->roles()->attach($role->id);
        } else {
            return response()->json(['status' => 'error', 'message' => 'No service role has been created.']);
        }
        return response()->json([
            'status' => 'success',
            'data' => $user
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    /*public function upload(Request $request)
    {
        $data = $request->data;
        $duplicate_error = [];
        $department_error = [];
        $insertions = [];

        foreach ($data as $dep => $service_names) {
            $department = Department::where(function ($query) use ($dep) {
                $query->whereRaw('LOWER(`name`) like ? ', array(strtolower($dep)));
            });

            if ($department->exists()) {
                $department = $department->first();
                foreach ($service_names as $service_name) {
                    $exists = Service::where('name', $service_name)->where('department_id', $department->id);
                    if ($exists->exists()) {
                        array_push($duplicate_error, ['name' => $service_name, 'department' => $department->name]);
                        continue;
                    }
                    $service = new Service();
                    $service->name = $service_name;
                    $service->department_id = $department->id;
                    $service->creator_id = Auth::id();
                    $service->status = true;

                    if ($service->save()) {
                        $insertions[] = $service_names;
                    }
                }
            } else {
                $department_error[] = ['name' => $dep, 'user' => $service_names];
                continue;
            }
        }
        $message = 'Service service upload complete.';
        return response()->json([
            'status' => 'success',
            'department_error' => $department_error,
            'duplicate_error' => $duplicate_error,
            'insertions' => $insertions,
            'message' => $message,
        ]);
    }*/

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param int $department_id
     * @return JsonResponse
     */
    public function updateDepartment(int $id, int $department_id): JsonResponse
    {
        $service = User::find($id);
        $service->department_id = $department_id;
        $service->update();

        return response()->json([
            'status' => 'success',
            'data' => $service
        ]);
    }


    /**
     * @param $id
     * @return JsonResponse
     */
    public function changeStatus($id)
    {
        $service = User::find($id);
        $role = Role::whereSlug('service')->first();
        if ($role) {
            $service->roles()->detach($role->id);
        }

        return response()->json([
            'status' => 'success'
        ]);
    }

    public function query(Request $request)
    {
        $params = $request->except(['__token']);
        $start = isset($params['date'][0]) ? $params['date'][0] : date('Y-m-d H:i:s', strtotime('1970-12-01 12:00:00'));
        $end = isset($params['date'][1]) ? $params['date'][1] : date('Y-m-d H:i:s');

        $data = Sale::with(['service', 'service.department'])
            ->where('created_at', '>=', $start)
            ->where('created_at', '<=', $end);
        if ($params['department']) {
            $data = $data->whereHas('user', function (Builder $query) use ($params) {
                return $query->where('department_id', '=', $params['department']);
            });
        }
        $data = $data->get();
        $push = [];
        foreach ($data as $sale) {
            if ($sale->service) {
                if (!isset($push[$sale->service_id])) {
                    $push[$sale->service_id] = [
                        'id' => $sale->service_id,
                        'name' => $sale->service->name,
                        'department' => $sale->service->department->name,
                        'total' => 0
                    ];
                }
                $push[$sale->service_id]['total'] += $sale->total_price;
            }
        }
        usort($push, function ($a, $b) {
            if ($a['total'] == $b['total']) {
                return 0;
            }
            return ($a['total'] > $b['total']) ? -1 : 1;
        });
        return array_values($push); // [$start, $end];
    }

    public function printServiceInventory(Request $request)
    {
        $sessions = $request->sessions;
        // $session = Session::find($sessions[0]);
        list($data, list($start_date, $end_date)) = $this->serviceInventory($sessions);

        $start_date = Carbon::parse($start_date)->toDayDateTimeString();
        $end_date = !$end_date ? null : Carbon::parse($end_date)->toDayDateTimeString();

        $grand_total = array_sum(array_map(function ($data) {
            return $data['total'];
        }, $data));

        $connector = new WindowsPrintConnector(env('RECEIPT_PRINTER'));
        $printer = new Printer($connector);

        $date = Carbon::now()->toDayDateTimeString();

        $printer->setJustification(Printer::JUSTIFY_CENTER);
        ### $printer -> bitImage($logo);
        /* Name of shop */
        $printer->selectPrintMode(Printer::MODE_DOUBLE_WIDTH);

        $printer->text(env('COMPANY_OWNER'));
        $printer->selectPrintMode();
        $printer->feed();
        $printer->text(env('COMPANY_NAME', 'SALAMANDA ERP'));
        $printer->feed();
        $printer->text(env('COMPANY_ADDRESS'));
        $printer->setFont(Printer::FONT_B);
        $printer->text(' (' . env('COMPANY_ADDRESS_SUB') . ')');
        $printer->setFont(Printer::FONT_A);
        $printer->feed();
        $printer->text(env('COMPANY_PHONE') . "\n");
        $printer->text(env('COMPANY_EMAIL') . "\n");

        $printer->feed(1);
        /* Title of receipt */
        $printer->setEmphasis(true);
        $printer->text("Service Receipt\n");
        $printer->setFont(Printer::FONT_B);
        $printer->setEmphasis(false);
        $printer->text("$start_date" . (!$end_date ? null : " - $end_date"));
        $printer->setFont(Printer::FONT_A);
        $max_chars = 55;

        $count = 1;
        usort($data, function ($a, $b) {
            if ($a['total'] == $b['total']) {
                return 0;
            }
            return ($a['total'] > $b['total']) ? -1 : 1;
        });
        // return $data;
        foreach ($data as $id => $service) {
            $printer->selectPrintMode();
            $printer->feed(2);
            $printer->setEmphasis(true);
            $printer->setJustification(Printer::JUSTIFY_LEFT);
            $printer->text("{$count}. {$service['name']}");
            $printer->setJustification(Printer::JUSTIFY_RIGHT);
            $amount = number_format($service['total'], 0, 0, $thousands_sep = ',');
            $pad_len = $max_chars - array_sum([strlen($count), strlen($service['name']), strlen($amount), 4]);
            $printer->text(str_pad("XAF {$amount}", $pad_len, " ", STR_PAD_LEFT));
            // $grand_total += $service['total'];
            $count++;
        }
        $printer->feed(2);
        $printer->setJustification(Printer::JUSTIFY_CENTER);
        $printer->setEmphasis();
        $printer->text("Grand Total: ");
        $printer->selectPrintMode(Printer::MODE_DOUBLE_WIDTH);
        $total = number_format($grand_total, 0, 0, ',');
        $printer->text("XAF {$total}");
        $printer->selectPrintMode();

        $printer->feed(2);
        $printer->text("Printed Account: " . \Illuminate\Support\Facades\Auth::user()->username);
        $printer->feed(1);
        $printer->text($date . "\n");
        /* Cut the receipt and open the cash drawer */
        $printer->cut();
        $printer->pulse();
        $printer->close();

    }

    public function serviceInventory($sessions)
    {
        $data = [];
        $dates = [];
        foreach (Session::whereIn('id', $sessions)->get() as $session) {
            $sales = $session->sales;
            // return $sales;
            foreach ($sales as $sale) {
                $service = $sale->service;
                if (!count($dates)) {
                    $dates[0] = $session->closed_on;
                    $dates[1] = count($sessions) > 1 ? $session->closed_on : null;
                }
                if ($service) {
                    // $data[] = ['name' => $service->name, 'amt' => $sale->total_price];
                    if (!isset($data[$service->id])) {
                        $data[$service->id] = ['name' => $service->name, 'total' => 0];
                    }
                    $data[$service->id]['total'] += (int)$sale->total_price;
                }
                if (count($sessions) > 1 && $session->closed_on->lt($dates[0])) {
                    $dates[0] = $session->closed_on;
                }
                if (count($sessions) > 1 && $session->closed_on->gt($dates[1])) {
                    $dates[1] = $session->closed_on;
                }
            }
        }
        return [$data, $dates];
    }

    public function printServiceQuery(Request $request)
    {
        $data = $this->query($request);
        $params = $request->except(['__token']);
        $start_date = isset($params['date'][0]) ? $params['date'][0] : date('Y-m-d H:i:s', strtotime('1970-12-01 12:00:00'));
        $end_date = isset($params['date'][1]) ? $params['date'][1] : date('Y-m-d H:i:s');

        $start_date = Carbon::parse($start_date)->toDayDateTimeString();
        $end_date = !$end_date ? null : Carbon::parse($end_date)->toDayDateTimeString();
        $grand_total = array_sum(array_map(function ($data) {
            return $data['total'];
        }, $data));
        $department = $params['department'] ? Department::find($params['department'])->name : 'All Departments';

        try {
            $connector = new WindowsPrintConnector("LVReceipt");
            $printer = new Printer($connector);

            $date = Carbon::now()->toDayDateTimeString();

            $printer->setJustification(Printer::JUSTIFY_CENTER);
            ### $printer -> bitImage($logo);
            /* Name of shop */
            $printer->selectPrintMode(Printer::MODE_UNDERLINE);
            $printer->selectPrintMode(Printer::MODE_DOUBLE_WIDTH);
            $printer->text(env('COMPANY_NAME', 'SALAMANDA ERP') . "\n");

            $printer->selectPrintMode();
            $printer->text('(BUEA - CAMEROON)');
            $printer->feed(2);

            $printer->selectPrintMode();
            $printer->text($department . ".\n");
            $printer->text(env('COMPANY_ADDRESS' . "\n"));
            $printer->text(env('COMPANY_PHONE') . "\n");
            $printer->text(env('COMPANY_EMAIL') . "\n");
            $printer->feed(2);
            /* Title of receipt */
            $printer->setEmphasis(true);
            $printer->text("Service Query Report\n");
            $printer->setFont(Printer::FONT_B);
            $printer->setEmphasis(false);
            $printer->text("$start_date - $end_date");
            $printer->setFont(Printer::FONT_A);
            $max_chars = 55;

            $count = 1;
            usort($data, function ($a, $b) {
                if ($a['total'] == $b['total']) {
                    return 0;
                }
                return ($a['total'] > $b['total']) ? -1 : 1;
            });
            // return $data;
            foreach ($data as $id => $service) {
                $printer->selectPrintMode();
                $printer->feed(2);
                $printer->setEmphasis(true);
                $printer->setJustification(Printer::JUSTIFY_LEFT);
                $printer->text("{$count}. {$service['name']}");
                $printer->setJustification(Printer::JUSTIFY_RIGHT);
                $amount = number_format($service['total'], 0, 0, $thousands_sep = ',');
                $pad_len = $max_chars - array_sum([strlen($count), strlen($service['name']), strlen($amount), 4]);
                $printer->text(str_pad("XAF {$amount}", $pad_len, " ", STR_PAD_LEFT));
                // $grand_total += $service['total'];
                $count++;
            }
            $printer->feed(3);
            $printer->setJustification(Printer::JUSTIFY_CENTER);
            $printer->setEmphasis();
            $printer->text("Grand Total: ");
            $printer->selectPrintMode(Printer::MODE_DOUBLE_WIDTH);
            $total = number_format($grand_total, 0, 0, ',');
            $printer->text("XAF {$total}");
            $printer->selectPrintMode();

            $printer->feed(2);
            $printer->text("Printed Account: " . Auth::user()->name);
            $printer->feed(2);
            $printer->text($date . "\n");
            /* Cut the receipt and open the cash drawer */
            $printer->cut();
            $printer->pulse();
            $printer->close();
        } catch (\Exception $e) {
            return response()->json([
                'status' => 'error',
                'message' => 'Printer error!'
            ]);
        }

        return \response()->json([
            'status' => 'success',
            'message' => 'Printing...'
        ]);
    }


}
