<?php

namespace App\Http\Controllers;

use App\Store;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;

class StoreController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function index(): JsonResponse
    {
        return response()->json([
            'status' => 'success',
            'data' => Store::all()
        ]);
    }

    /**
     * @throws ValidationException
     */
    public function store(Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => 'required',
            'address' => 'required'
        ]);

        $store = new Store();
        $store->name = $request->input('name');
        $store->address = $request->input('address');
        $store->phone = $request->input('phone');
        $store->description = $request->input('description');
        $store->user_id = auth()->id();
        $store->code = get_next_ref('store');

        $store->save();

        return response()->json([
            'status' => 'success',
            'data' => $store->refresh()
        ]);
    }

    /**
     * @throws ValidationException
     */
    public function update(Store $store, Request $request): JsonResponse
    {
        $this->validate($request, [
            'name' => 'required',
            'address' => 'required'
        ]);

        $store->name = $request->input('name');
        $store->address = $request->input('address');
        $store->phone = $request->input('phone');
        $store->description = $request->input('description');

        $store->update();

        return response()->json([
            'status' => 'success',
            'data' => $store->refresh()
        ]);
    }

    /**
     * @throws ValidationException|Exception
     */
    public function delete(Store $store): JsonResponse
    {
        $store->delete();

        return response()->json([
            'status' => 'success'
        ]);
    }
}
