<?php

namespace App\Http\Controllers;

use App\Supplier;
use App\SupplierPayment;
use App\Supply;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class SupplierController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    public function getSuppliers()
    {
        return response()->json([
            'status' => 'success',
            'data' => Supplier::whereState(true)
                ->with(['user'])
                ->withCount('supplies')
                ->latest()->get(),
            'lang' => trans('auth.suspended')
        ], 200);
    }

    public function getSupplier($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => Supplier::withCount(['payments' => function ($query) {
                return $query->select(DB::raw("SUM(amount)"))
                    ;
            }])->find($id)
        ], 200);
    }

    public function createSupplier(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'phone' => 'required|size:9',
            'address' => 'required'
        ], [
            'name.required' => trans('client_validation.supplier.name.required')
        ]);

        $supplier = new Supplier();
        $supplier->name = $request->input('name');
        $supplier->phone = $request->input('phone');
        $supplier->address = $request->input('address');
        $supplier->user_id = auth()->id();

        $supplier->save();
        return response()->json([
            'status' => 'success',
            'data' => $supplier
        ], 200);
    }

    public function updateSupplier($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'phone' => 'required|size:9',
            'address' => 'required'
        ], [
            'name.required' => 'Please, provide a valid name.'
        ]);

        $supplier = Supplier::find($id);
        $supplier->name = $request->input('name');
        $supplier->address = $request->input('address');
        $supplier->phone = $request->input('phone');

        $supplier->update();
        return response()->json([
            'status' => 'success',
            'data' => $supplier
        ], 200);
    }

    public function archiveSupplier($id)
    {
        $supplier = Supplier::find($id);
        $supplier->state = !$supplier->state;
        $supplier->save();

        return response()->json([
            'status' => 'success',
            'state' => $supplier->state
        ], 200);
    }

    public function getArchiveSuppliers()
    {
        return response()->json([
            'status' => 'success',
            'data' => Supplier::where('state', 0)->get(),
        ], 200);
    }

    public function supplierHistory($id)
    {
        $history = Supply::whereSupplierId($id)
            ->with(['items.item', 'user'])
            ->withCount(['items'])
            ->latest()
            ->get();
        return response()->json([
            'status' => 'success',
            'data' => $history
        ]);
    }

    public function getSupplierPayment($id)
    {
        $payment = SupplierPayment::where('supplier_id', $id)->with(['user'])->latest()->get();

        return response()->json([
            'status' => 'success',
            'data' => $payment
        ], 200);
    }

    public function cashIn($id, Request $request)
    {
        $this->validate($request, [
            'amount' => 'required|numeric',
            'message' => 'required|string'
        ]);

        $supplierPayment = new SupplierPayment();
        $supplierPayment->user_id = Auth::id();
        $supplierPayment->supplier_id = $id;
        $supplierPayment->amount = 0;

        $supplierPayment->message = "Direct debt (-" . number_format($request->amount) . ' XAF): ' . $request->message;
        $supplierPayment->save();

        $supplier = Supplier::find($id);
        $supplier->amt_owed += $request->amount;
        $supplier->update();

        return response()->json([
            'status' => 'success',
            'data' => $supplier
        ], 200);
    }
}
