<?php

namespace App\Http\Controllers;

use App\AccountSubcategory;
use App\Supplier;
use App\SupplierPayment;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class SupplierPaymentController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function getSupplierPayments()
    {
        return response()->json([
            'status' => 200,
            'data' => SupplierPayment::latest()->get()
        ], 200
        );
    }

    public function getSupplierPayment($id)
    {
        return response()->json([
            'status' => 'success',
            'data' => SupplierPayment::find($id)
        ], 200);
    }

    public function createSupplierPayment(Request $request)
    {
        $this->validate($request, [
            'supplier_id' => 'required',
            'amount' => 'required',
            'message' => 'required',
            'date' => 'required|date'
        ], [
            'amount.required' => 'Please Provide the amount.',
        ]);

        $supplier_id = $request->input('supplier_id');
        $amount_to_be_paid = $request->input('amount');
        $amount_owed = Supplier::find($supplier_id)->amt_owed;

        if (intval($amount_to_be_paid) > $amount_owed + 24) {
            return response()->json([
                'status' => 'error',
                'message' => 'Please make sure payment amount is not greater than total price + 24FCFA'
            ], 403);
        }

        $account_sub_category = AccountSubcategory::where('slug', 'like', '%supplier%')->first();
        if (!$account_sub_category) {
            return response()->json([
                'status' => 'error',
                'message' => "Please make sure an account subcategory exists called `...supplier...`.
                This subcategory will be used to register this payment."
            ], 403);
        }

        $amount = $amount_to_be_paid > $amount_owed ? $amount_owed : $amount_to_be_paid;
        $supplierPayment = new SupplierPayment();
        $supplierPayment->user_id = Auth::id();
        $supplierPayment->supplier_id = $supplier_id;
        $supplierPayment->amount = $amount;
        $supplierPayment->message = $request->message;
        $supplierPayment->save();

        $supplier = Supplier::find($supplier_id);
        $supplier->decrement('amt_owed', $amount);

        $withdraw = new WithdrawalController();
        $req = $request;
        $req->request->add([
            'message' => "Paying for supplier {$supplier->name}",
            'subcategory_id' => $account_sub_category->id
        ]);
        $withdraw->makeWithdrawal($req);

        return response()->json([
            'status' => 'success',
            'data' => SupplierPayment::with('user')->find($supplierPayment->id)
        ]);
    }

    public function updateSupplierPayment($id, Request $request)
    {
        $this->validate($request, [
            'user_id' => 'required',
            'supplier_id' => 'required',
            'amount' => 'required',
        ], [
            'amount.required' => 'Please Provide the amount.',
        ]);

        $supplierPayment = SupplierPayment::find($id);
        $supplierPayment->amount = $request->input('amount');

        $supplierPayment->save();
        return response()->json([
            'status' => 'success',
            'data' => $supplierPayment
        ]);
    }
}
