<?php

namespace App\Http\Controllers;

use App\Transfer;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class TransferController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function fetchTransfers(Request $request)
    {
        $query = Transfer::query();
        if ($request->filled('start_date')) {
            $query = $query->whereDate('created_at', '>=', $request->get('start_date'));
        }
        if ($request->filled('end_date')) {
            $query = $query->whereDate('created_at', '<=', $request->get('end_date'));
        }
        if ($request->filled('from_warehouse') && count($request->get('from_warehouse'))) {
            $query = $query->whereIn('from_id', $request->get('from_warehouse'));
        }
        if ($request->filled('to_warehouse') && count($request->get('to_warehouse'))) {
            $query = $query->whereIn('to_id', $request->get('to_warehouse'));
        }
        if ($request->filled('item') && count($request->get('item'))) {
            $query = $query->whereIn('item_id', $request->get('item'));
        }
        if ($request->filled('user') && count($request->get('user'))) {
            $query = $query->whereIn('user_id', $request->get('user'));
        }
        return response()->json([
            'status' => 'success',
            'data' => $query->with('from', 'to', 'item', 'user')
                ->latest()
                ->get()
                ->groupBy(function ($item) {
                    return $item->created_at->format('d-M-y');
                })
        ]);
    }

    public function executeTransfers(Request $request): JsonResponse
    {

        $this->validate($request, [
            'data' => 'array|min:1'
        ]);

        DB::transaction(function () use ($request) {
            $data = $request->get('data');
            foreach ($data as $datum) {
                $whCtrl = new WarehouseController();
                $whCtrl->transferArticle(
                    $items = [['id' => $datum['item'], 'quantity' => $datum['quantity']]],
                    $to_id = $datum['to_warehouse'],
                    $from_id = $datum['from_warehouse'],
                    $comment = $request->get('comment')
                );
            }
        });

        return response()->json([
            'status' => 'success'
        ]);
    }

}
