<?php

namespace App\Http\Controllers;

use App\Item;
use App\ItemVariant;
use App\Variant;
use App\VariantOption;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class VariantController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }


    public function index()
    {
        $variants = Variant::with(['user', 'item'])->get();
        return response()->json([
            'status' => 'success',
            'data' => $variants
        ]);
    }

    public function create(Request $request, $item_id): JsonResponse
    {
        $variants = $request->all();
        $item_variants = $this->save_variant($variants, $item_id);

        return response()->json([
            'status' => 'success',
            'data' => $item_variants
        ]);
    }

    public function save_variant($variants, $item_id)
    {
        $item = Item::find($item_id);
        $data = [];

        foreach ($variants as $index => $variant) {
            $new_variant = new Variant();
            $new_variant->name = $variant['name'];
            $new_variant->item_id = $item->id;
            $new_variant->user_id = auth()->id();
            $new_variant->save();
            $data[$index] = [];

            foreach ($variant['options'] as $key => $option) {
                $new_option = new VariantOption();
                $new_option->name = $option;
                $new_option->variant_id = $new_variant->id;
                // $new_option->user_id = auth()->id();
                $new_option->save();

                $data[$index][] = ["id" => $new_option->id, "name" => $new_option->name];
            }
        }
        $combinations = $this->generateCombinations($data);

        foreach ($combinations as $combination) {
            $item_variant = new ItemVariant();
            $item_variant->item_id = $item->id;
            $item_variant->unit_price = $item->unit_price ?? 0; // @todo get price from price_list
            $item_variant->purchase_price = $item->purchase_price;
            $name = '';
            for ($i = 0; $i < count($combination); $i++) {
                $item_variant["option_" . ($i + 1)] = $combination[$i]['id'];
                $name .= (($i == count($combination) - 1) ? $combination[$i]['name'] : ($combination[$i]['name'] . " / "));
            }
            $item_variant->name = $name;
            $item_variant->code = get_next_ref('variant');
            $item_variant->barcode = 0; // create barcode
            $item_variant->save();
        }

        return ItemVariant::where('item_id', $item_id)->get();
    }

    function generateCombinations(array $arrays): array
    {
        if (empty($arrays)) {
            return [[]];
        }

        $firstArray = $arrays[0];
        $restArrays = array_slice($arrays, 1);

        $combinations = [];
        foreach ($firstArray as $element) {
            $subCombinations = $this->generateCombinations($restArrays);
            foreach ($subCombinations as $subCombination) {
                $combinations[] = array_merge([$element], $subCombination);
            }
        }

        return $combinations;
    }

    public function get_item_variant($item_id): JsonResponse
    {
        return response()->json([
            'status' => 200,
            'data' => Variant::with(['options'])->where('item_id', $item_id)->get()
        ]);
    }

    public function store(Request $request)
    {

        $this->validate($request, [
            'name' => 'required',
            'item_id' => 'required|numeric'
        ]);

        $variant = new Variant();
        $variant->name = $request->input('name');
        $variant->item_id = $request->input('item_id');
        $variant->user_id = auth()->id();
        $variant->save();

        return response()->json([
            'status' => 'success',
            'data' => $variant->refresh()
        ]);
    }

    public function destroy($id)
    {
        $variant = Variant::find($id);
        $variant->delete();
        return response()->json([
            'status' => 'success',
            'data' => $variant
        ], 200);
    }

    public function getOptions()
    {
        $variant_options = VariantOption::with('variant')->get();
        return response()->json([
            'status' => 'success',
            'data' => $variant_options
        ]);
    }

    public function getVariantOptions($variant_id)
    {
        $options = VariantOption::with('variant')->get();
        $variant_options = [];
        foreach ($options as $option) {
            if ($option->variant_id == $variant_id) {
                $variant_options[] = $option;
            }
        }
        return response()->json([
            'status' => 'success',
            'data' => $variant_options
        ]);
    }

    public function storeOption(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'variant_id' => 'required|numeric'
        ]);

        $variant_option = new VariantOption();
        $variant_option->name = $request->input('name');
        $variant_option->variant_id = $request->input('variant_id');
        // $variant_option->user_id = auth()->id();
        $variant_option->save();

        return response()->json([
            'status' => 'success',
            'data' => $variant_option->refresh()
        ]);
    }

    public function updateOption($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'variant_id' => 'required|numeric'
        ]);
        $variant_option = VariantOption::where('id', $id)->first();
        $variant_option->update($request->all());

        return response()->json([
            'status' => 'success',
            'data' => $variant_option->refresh()
        ]);

    }

    public function update($id, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'item_id' => 'required|numeric'
        ]);
        $variant = Variant::where('id', $id)->first();
        $variant->update($request->all());

        return response()->json([
            'status' => 'success',
            'data' => $variant->refresh()
        ]);

    }

    public function destroyOption($id)
    {
        $variant_option = VariantOption::find($id);
        $variant_option->delete();
        return response()->json([
            'status' => 'success',
            'data' => $variant_option
        ], 200);
    }
}
