<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Item extends Model
{
    const defaultColour = "4279858898";
    protected $casts = [
        'sellable' => 'boolean',
        'reorder_level' => 'integer',
        'optimal_order' => 'double',
        'state' => 'boolean'
    ];

    public function user()
    {
        return $this->belongsTo(User::class)->select('id', 'name', 'username');
    }

    public function scopeActive($query, $state = 'state')
    {
        return $query->where($state, true);
    }

    public function category(): BelongsTo
    {
        return $this->belongsTo(Category::class)->select('id', 'name', 'department_id', 'tracked', 'user_id');
    }

    public function getImageAttribute($value): string
    {
        return strlen($value) ? url($value) : 'https://picsum.photos/id/1/200/300';
    }

    public function price_lists(): BelongsToMany
    {
        return $this->belongsToMany(PriceList::class)->withPivot(['price']);
    }

    public function warehouses(): BelongsToMany
    {
        return $this->belongsToMany(Warehouse::class, 'stocks')->withPivot('quantity', 'selling_price', 'state');
    }

    public function modifiers(): BelongsToMany
    {
        return $this->belongsToMany(Modifier::class);
    }

    public function variants(): HasMany
    {
        return $this->hasMany(ItemVariant::class); // @updated
    }

    public function item_variants()
    {
        return $this->hasMany(ItemVariant::class);
    }

    public function taxes(): BelongsToMany
    {
        return $this->belongsToMany(Tax::class); // @updated
    }
}
