<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class SalesOrder extends Model
{
    use SoftDeletes;

    protected $table = 'sales_orders';

    protected $fillable = ['owner_id', 'comment'];

    protected $casts = [
        'status' => 'integer'
    ];

    public function items()
    {
        return $this->hasMany(SalesOrderItem::class); ### ->with('invoice');
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'owner_id');
        /*->whereHas('department', function ($query) {
            $query->where('sales_orders.department', 'users.department_id');
        });*/
    }

    public function cashier()
    {
        return $this->belongsTo(User::class, 'owner_id');
        /*->whereHas('department', function ($query) {
            $query->whereColumn('sales_order.department_id', 'users.department_id');
        });*/
    }

    public function creator()
    {
        return $this->belongsTo(User::class, 'creator_id');
    }

    public function owner()
    {
        return $this->belongsTo(User::class, 'owner_id');
    }

    public function service()
    {
        return $this->belongsTo(User::class, 'service_id')->with(['department' => function ($query) {
            $query->select('id', 'name');
        }]);
    }

    public function customer()
    {
        return $this->belongsTo(Customer::class);
    }

    public function session()
    {
        return $this->belongsTo(Session::class);
    }

    public function department()
    {
        return $this->belongsTo(Department::class);
    }

    public function scopeSessionOrders($query, $session_id)
    {
        return $query->where('session_id', $session_id);
    }

    public function scopeDepartmentOrders($query)
    {
        return $query->where('department_id', auth()->user()->department_id);
    }

    public function scopeOwnerOrders($query)
    {
        if (auth()->user()->hasRole('cashier')) {
            return $query->departmentOrders();
        }

        if (auth()->user()->hasRole('service')) {
            return $query->where('owner_id', auth()->id());
        }
        return $query;
    }

    public function scopePendingOrders($query)
    {
        return $query->where('status', '0');
    }

    public function scopeChargedOrders($query)
    {
        return $query->where('status', '1');
    }

    public function scopeRevokedOrders($query)
    {
        return $query->where('status', '-1');
    }
}
