<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class StockAdjustment extends Model
{
    protected $fillable = ['warehouse_id', 'reason', 'date', 'user_id'];
    protected $with = ['user'];
    protected $appends = ['summary'];

    public function getSummaryAttribute()
    {
        $items = collect($this->items()->whereColumn('from', '!=', 'to')->get());
        // $lang_en = auth()->user()->lang == 'en';
        $data = [
            'total_purchase_price' => $items->sum(function ($item) {
                return $item['avr_purchase_price'] * ($item['from'] - $item['to']);
            }),
            'total_sales' => $items->sum(function ($item) {
                return $item['unit_price'] * ($item['from'] - $item['to']);
            })
        ];
        $data['total_profit'] = $data['total_sales'] - $data['total_purchase_price']; // number_format(, 2, ($lang_en ? '.' : ','), ($lang_en ? ',' : '.'));

        return $data;
    }

    public function items(): HasMany
    {
        return $this->hasMany(ItemStockAdjustment::class); // ->orderByRaw('(to - from) DESC');
    }

    public function warehouse(): BelongsTo
    {
        return $this->belongsTo(Warehouse::class);
    }

    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class)->select(['id', 'name', 'username', 'photo', 'photo_thumbnail']);
    }
}
