<?php

/** @var \Illuminate\Database\Eloquent\Factory $factory */

use App\AccountCategory;
use App\AccountSubcategory;
use App\Debt;
use App\DebtType;
use App\Department;
use App\Employee;
use App\EmployeeType;
use App\Item;
use App\POS;
use App\Store;
use App\Supply;
use App\User;
use Faker\Generator as Faker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use App\Supplier;

/*
|--------------------------------------------------------------------------
| Model Factories
|--------------------------------------------------------------------------
|
| This directory should contain each of the model factory definitions for
| your application. Factories provide a convenient way to generate new
| model instances for testing / seeding your application's database.
|
*/

$factory->define(Store::class, function (Faker $faker) {
    return [
        'name' => $faker->name,
        'description' => $faker->sentences(2, true),
        'address' => $faker->address,
        'phone' => $faker->phoneNumber,
        'code' => $faker->uuid,
        'user_id' => 1,
    ];
});

$factory->define(POS::class, function (Faker $faker) {
    return [
        'name' => $faker->company,
        'code' => $faker->uuid,
        'user_id' => 1,
        'store_id' => random_int(1, 3)
    ];
});

$factory->define(User::class, function (Faker $faker) {
    return [
        'name' => $faker->name,
        'email' => $faker->email,
    ];
});

$factory->define(App\Supplier::class, function (Faker $faker) {
    return [
        'name' => $faker->name,
        'phone' => $faker->randomNumber(9),
        'address' => $faker->address,
        'user_id' => 1,
    ];
});

$factory->define(App\Item::class, function (Faker $faker) {
    return [
        'name' => $faker->company,
        'unit_price' => $faker->randomNumber(4),
        'unit_type' => $faker->colorName,
        'category_id' => $faker->numberBetween(1, 3),
        'user_id' => 1,
    ];
});
$factory->define(App\Category::class, function (Faker $faker) {
    return [
        'name' => $faker->colorName,
        'department_id' => $faker->numberBetween(1,3),
        'user_id' => 1,
    ];
});

$factory->define(App\Supply::class, function (Faker $faker) {
    return [
        'supplier_id' => Supplier::all('id')->random(),
        'user_id' => 1,
        'total_cost' => $faker->randomFloat(3, 20000, 120000),
        'paid' => $faker->randomFloat(3, 20000, 120000),
        'owed' => $faker->randomFloat(3, 200, 40234),
    ];
});

$factory->define(App\ItemSupply::class, function (Faker $faker) {
    return [
        'supply_id' => Supply::all('id')->random(),
        'item_id' => Item::all('id')->random(),
        'quantity' => $faker->randomNumber(3),
        'unit_price' => $faker->randomFloat(3, 200, 40000),
        'total_cost' => $faker->randomFloat(3, 50000, 1000000),
    ];
});

$factory->define(App\SupplierPayment::class, function (Faker $faker) {
    return [
        'user_id' => 1,
        'supplier_id' => Supplier::all('id')->random(),
        'amount' => $faker->randomFloat(3, 10000, 40000)
    ];
});

//$factory->define(App\Warehouse::class, function (Faker $faker) {
//    return [
//        'item_id' => Item::all('id')->random(),
//        'quantity' => $faker->randomNumber(3),
//        'unit_type' => $faker->randomLetter
//
//    ];
//});

$factory->define(App\Account::class, function (Faker $faker) {
    return [
        'amount' => $faker->randomNumber(7),
    ];
});

$factory->define(App\AccountCategory::class, function (Faker $faker) {
    return [
        'name' => $faker->company
    ];
});

$factory->define(App\AccountSubcategory::class, function (Faker $faker) {
    return [
        'name' => $faker->company,
        'account_category_id' => AccountCategory::all('id')->random(),
        'amount' => $faker->randomNumber(6)
    ];
});

$factory->define(App\Deposit::class, function (Faker $faker) {
    return [
        'account_subcategory_id' => AccountSubcategory::all('id')->random(),
        'amount' => $faker->randomNumber(6),
        'message' => $faker->text(),
        'date' => $faker->date('Y_m_d'),
        'user_id' => 1
    ];
});

$factory->define(App\Withdrawal::class, function (Faker $faker) {
    return [
        'account_subcategory_id' => AccountSubcategory::all('id')->random(),
        'amount' => $faker->randomNumber(6),
        'message' => $faker->text(),
        'date' => $faker->date('Y-m-d'),
        'user_id' => 1
    ];
});

$factory->define(App\EmployeeType::class, function (Faker $faker) {
    return [
        'name' => $faker->realText(10),
        'salary' => $faker->randomNumber(6),
        'description' => $faker->sentence,
        'user_id' => 1
    ];
});

$factory->define(App\Employee::class, function (Faker $faker) {
    return [
        'name' => $faker->firstName,
        'phone' => $faker->phoneNumber,
        'email' => $faker->email,
        'nic' => $faker->randomNumber(9),
        'address' => $faker->address,
        'state' => $faker->boolean(),
        'type_id' => EmployeeType::all('id')->random(),
        'user_id' => User::all('id')->random(),
        'employment_date' => $faker->date(),
    ];
});


$factory->define(App\DebtType::class, function (Faker $faker) {
    return [
        'name' => $faker->realText(20),
        'can_pay' => $faker->boolean(50),
        'affects_account' => $faker->boolean(50),
        'user_id' => 1
    ];
});

$factory->define(App\Debt::class, function (Faker $faker) {
    return [
        'employee_id' => Employee::all('id')->random(),
        'type_id' => DebtType::all('id')->random(),
        'amount' => $faker->randomNumber(5),
        'description' => implode(" ", $faker->sentences(2)),
        'user_id' => 1,
        'date' => $faker->date(),
    ];
});

//$factory->define(App\Deposit::class, function (Faker $faker) {
//    return [
//        'account_subcategory_id' => AccountSubcategory::all('id')->random(),
//        'amount' => $faker->randomNumber(5),
//        'message' => $faker->sentence,
//        'user_id' => 1
//    ];
//});

//$factory->define(App\Withdrawal::class, function (Faker $faker) {
//    return [
//        'account_subcategory_id' => AccountSubcategory::all('id')->random(),
//        'amount' => $faker->randomNumber(5),
//        'message' => $faker->sentence,
//        'user_id' => 1
//    ];
//});

$factory->define(App\Department::class, function (Faker $faker) {
    return [
        'name' => $faker->realText(),
        'user_id' => 1
    ];
});

$factory->define(App\Stock::class, function (Faker $faker) {
    return [
        'item_id' => Item::all('id')->random(),
        'department_id' => Department::all('id')->random(),
        'quantity' => $faker->randomNumber(2)
    ];
});

$factory->define(App\Transfer::class, function (Faker $faker) {
    return [
        'item_id' => Item::all('id')->random(),
        'quantity' => $faker->randomNumber(2),
        'from_id' => Department::all('id')->random(),
        'to_id' => Department::all('id')->random(),
        'user_id' => 1
    ];
});

$factory->define(App\EmployeePayment::class, function (Faker $faker) {
   return [
       'employee_id' => Employee::all('id')->random(),
       'amount' => $faker->randomNumber(3),
       'message' => $faker->sentence,
       'date' => $faker->date(),
       'user_id' => 1
   ];
});

$factory->define(App\Salary::class, function(Faker $faker) {
    return [
        'employee_id' => Employee::all('id')->random(),
        'total_salary' => $faker->randomNumber(4),
        'debt' => $faker->randomNumber(4),
        'paid' => $faker->randomNumber(3),
        'user_id' => 1,
        'date' => $faker->date()
    ];
});
