<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateSalesOrdersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('sales_orders', function (Blueprint $table) {
            // Maintained fields 1
            $table->increments('id');
            $table->string('code');
            $table->enum('status', [0, 1, -1])->default(0); // 0: Created, 1: Charged, -1: Revoked
            $table->unsignedInteger('session_id')->nullable();

            // Removed fields
            $table->unsignedInteger('service_id')->nullable(); // becomes either owner_id or created_by. Service is a user and can own
            $table->integer('user_id')->unsigned(); // becomes either owner_id or created_by

            /*
            // Added fields (check related migration file created `update_sales_orders_table`
            $table->string('label')->nullable();
            $table->unsignedInteger('owner_id');
            $table->unsignedInteger('creator_id')->nullable();
            $table->unsignedInteger('department_id')->nullable();
            $table->double('total')->default(0);
            $table->string('comment')->nullable();
            */

            // Maintained fields 2
            $table->unsignedInteger('customer_id')->nullable();
            $table->double('discount')->default(0);
            $table->timestamps();
            $table->softDeletes();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('sales_orders');
    }
}
