<?php

use App\Permission;
use App\Role;
use Illuminate\Database\Seeder;

class PermissionsTableSeeder extends Seeder
{
    /**
     * Run the database seeds.
     *
     * @return void
     */
    public function run()
    {
        $maps = ['a' => 'assign', 'dept' => 'department', 'a-p' => 'assign-permission', 'c' => 'create', 'd' => 'delete', 'r' => 'read', 'u' => 'update', 's' => 'sync', 'cl' => 'close', 'p' => 'print', 'ps' => 'print-service', 'rp' => 'reprint', 'tr' => 'transfer', 'hs' => 'history', 'df' => 'default', 'mn' => 'manage', 'aj' => 'adjust', 'pay' => 'payment', 'type' => 'type', 'r-pass' => 'reset-password', 'rc' => 'read-category', 'cc' => 'create-category', 'uc' => 'update-category', 'dc' => 'delete-category', 'rsc' => 'read-subcategory', 'csc' => 'create-subcategory', 'usc' => 'update-subcategory', 'dsc' => 'delete-subcategory', 'ar' => 'archive', 'ars' => 'archives', 'wh' => 'warehouse', 'pl' => 'price-list', 'stock' => 'stock', 'supply' => 'supply', 'spayment' => 'supplier-payment', 'p-r' => 'payroll', 'debt' => 'debt', 'deposit' => 'deposit', 'with' => 'withdrawal', 'edebt' => 'employee-debt', 'inv' => 'inventory', 'rec' => 'receipt', 'df-msg' => 'default-message', 'org' => 'organisation', 'reg' => 'registration'];
        $modules = [
            'user' => 'c,r,u,d,ar,r-pass',
            'item' => 'c,r,u,d,ar,ars,stock',
            'invoice' => 'c,r,u,d',
            'dashboard' => 'r',
            'role' => 'c,r,u,d,a,a-p',
            'session' => 'c,r,cl,p,ps',
            'permission' => 'a',
            'category' => 'c,r,u,d,ar',
            'department' => 'c,r,u,d,ar,wh',
            'payment_method' => 'c,r,u,d',
            'report' => 'r,supply,spayment,edebt,deposit,p-r,inv,with,p-r,spayment',
            'supply' => 'c,r',
            'service' => 'c,r,u,d,ar,dept',
            'supplier' => 'c,r,u,d,hs,pay,ar,ars',
            'employee' => 'c,r,u,d,ar,ars,type',
            'customer' => 'c,r,u,d,ar,ars,pl',
            'sales_order' => 'c,r,u,d',
            'debt' => 'c,r,pay,type',
            'payroll' => 'c,r',
            'deposit' => 'c,r',
            'withdrawal' => 'c,r',
            'warehouse' => 'c,r,u,d,hs,tr,mn',
            'account' => 'rc,cc,uc,dc,rsc,csc,usc,dsc',
            'stock' => 'aj',
            'setting' => 'r,pl,rec,df-msg,org,reg'
        ];

        $inserts = [];
        foreach ($modules as $name => $module) {
            foreach (explode(',', $module) as $value) {
                $this->command->info("Creating {$maps[$value]}.{$name} permission");
                $inserts[] = [
                    'slug' => "{$name}.{$maps[$value]}",
                    'name' => ucfirst($maps[$value]) . ' ' . ucfirst($name),
                    'description' => ucfirst($maps[$value]) . ' ' . ucfirst($name)
                ];
            }
        }
        Permission::truncate();
        Permission::insert($inserts);
        $role = Role::whereSlug('cashier');
        // Assign cashier permissions if exists
        if ($role->exists()) {
            $permissions = Permission::whereIn('slug', ['invoice.create', 'invoice.read', 'invoice.update', 'invoice.delete', 'session.close'])->pluck('id');
            $role->first()->permissions()->sync($permissions);;
        }
    }
}
