<?php

/** @var \Laravel\Lumen\Routing\Router $router */

/*
|--------------------------------------------------------------------------
| Application Routes
|--------------------------------------------------------------------------
|
| Here is where you can register all of the routes for an application.
| It is a breeze. Simply tell Lumen the URIs it should respond to
| and give it the Closure to call when that URI is requested.
|
*/


use App\SalesOrder;
use App\User;
use Illuminate\Support\Facades\Route;
use Mike42\Escpos\PrintConnectors\WindowsPrintConnector;
use Mike42\Escpos\Printer;


Route::get('/see', function () {
    return "I See You!!!";
});

Route::get('/storage', function () {
    \Illuminate\Support\Facades\Storage::link();
});

Route::get('/print', function () {
    $connector = new WindowsPrintConnector(env('RECEIPT_PRINTER'));
    $printer = new Printer($connector);

    $printer->text("Hello, World!");
    $printer->cut();
    $printer->close();
    /*return array_map(function ($query) {
        return [$query['id'] => ['amount' => $query['amount']]];
    }, [['id' => 2, 'amount' => 13000], ['id' => 3, 'amount' => 21000]]);*/
    \Illuminate\Support\Facades\Auth::attempt(['username' => 'maggi', 'password' => 'password']);
    // return SalesOrder::ownerOrders()->pendingOrders()->with(['items', 'service', 'user'])->get();
});

Route::get('test', function () {
    return User::find(2)->department->warehouse->items()->with(['category', 'price_lists'])->get();//\App\User::with(['department.warehouse.items.category', 'department.warehouse.items.price_lists'])->find(2);
    // return \App\User::find(2)->department->warehouse->items()->with(['category', 'price_lists'])->get();
    // app(SalesOrder::class)->getTable(); // \App\Department::find(1)->sales_orders()->revokedOrders()->get();
});

$router->group(['prefix' => 'api/v1'], function () use ($router) {

    Route::group(['prefix' => 'license'], function () {
        Route::post('/setup', 'LicenseController@setup_license');
        Route::get('/verify', 'LicenseController@verify_license');
    });

    // Ping Server
    Route::post('/ping', function () {
        return response()->json(['status' => 200, 'data' => 'pong']);
    });

    // Store Routes
    Route::group(['prefix' => 'stores', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'StoreController@index');
        Route::post('store', 'StoreController@store');
        Route::post('update/{store:[0-9]+}', 'StoreController@update');
        Route::post('delete/{store:[0-9]+}', 'StoreController@delete');
    });

    // POS Routes
    Route::group(['prefix' => 'poses', 'middleware' => 'auth:api'], function () {
        Route::get('', 'POSController@index');
        Route::get('/{id:[0-9]+}', 'POSController@index');
        Route::post('store', 'POSController@store');
        Route::post('update/{pos:[0-9]+}', 'POSController@update');
        Route::post('status/{pos:[0-9]+}', 'POSController@change_status');
        Route::post('delete/{pos:[0-9]+}', 'POSController@delete');
    });

    Route::group(['prefix' => 'reports'], function () {
        Route::get('/', 'ReportController@index');
        Route::get('/supplier/payment', 'ReportController@getSupplierPayment');
        Route::get('/expected-expenditure', 'ReportController@expectedExpenditure');
        Route::get('/employee-debts', 'ReportController@employeeDebts');
        Route::get('/supplies', 'ReportController@getSupplies');
        Route::get('/deposits', 'ReportController@deposits');
        Route::get('/withdrawals', 'ReportController@withdrawals');
        Route::get('/accounts-transaction', 'ReportController@accountTransactions');
        Route::get('/inventories', 'ReportController@getInventory');
    });

    $router->get('/', function () use ($router) {
        return $router->app->version();
    });

    //  Auth Routes
    $router->group(['prefix' => 'auth'], function () {
        Route::post('login', 'AuthController@login');
        Route::post('password/confirm', 'AuthController@verifyPassword');

        Route::group(['middleware' => 'auth:api'], function () {
            Route::post('logout', 'AuthController@logout');
            Route::post('refresh', 'AuthController@refresh');
            Route::post('me', 'AuthController@me');
            Route::post('password', 'AuthController@changePassword');
            Route::post('language', 'AuthController@changeLanguage');
        });
    });

    Route::group(['prefix' => 'config', 'middleware' => 'auth:api'], function () {
        Route::get('', 'ConfigController@getConfig');
        Route::post('', 'ConfigController@setConfig');
    });

    //  Suppliers Routes
    Route::group(['prefix' => 'suppliers', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SupplierController@getSuppliers');
        Route::get('/{id:[0-9]+}', 'SupplierController@getSupplier');
        Route::post('/create', 'SupplierController@createSupplier');
        Route::post('/update/{id:[0-9]+}', 'SupplierController@updateSupplier');

        Route::post('/{id:[0-9]+}/state', 'SupplierController@changeState');

        Route::post('/{id:[0-9]+}/archive', 'SupplierController@archiveSupplier');
        Route::get('/archive/all', 'SupplierController@getArchiveSuppliers');
        Route::post('/archive/{id:[0-9]+}/restore', 'SupplierController@changeState');
        Route::get('/{id:[0-9]+}/history', 'SupplierController@supplierHistory');
        Route::post('/{id:[0-9]+}/cash-in', 'SupplierController@cashIn');
        Route::get('/{id:[0-9]+}/help', function ($id) {
            return response()->json([
                'status' => 'success',
                'data' => App\Supply::whereSupplierId($id)->sum('paid')
            ], 200);
        });
        Route::get('/payments/{id:[0-9]+}', 'SupplierController@getSupplierPayment');
    });

    // Item Routes
    Route::group(['prefix' => 'items', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ItemController@getItems');
        Route::get('/active', 'ItemController@getActiveItems');
        Route::get('/categories', 'ItemController@getItemCategories');
        Route::get('/pos-categories', 'ItemController@getPOSItems');
        Route::get('/suggestions/unit-types', 'ItemController@getItemsUnitTypes');
        Route::get('/{id:[0-9]+}', 'ItemController@getItem');
        Route::post('/create', 'ItemController@createItem');
        Route::post('/{id:[0-9]+}/update', 'ItemController@updateItem');
        Route::post('/{id:[0-9]+}/archive', 'ItemController@changeState');
        Route::get('/archive/all', 'ItemController@getArchivedItems');
        Route::post('/archive/{id:[0-9]+}/restore', 'ItemController@changeState');
        Route::post('/mass-insert', 'ItemController@massInsert');
        Route::post('/next-ski', 'ItemController@nextRef');

        Route::post('/next-ski', 'ItemController@nextRef');

        Route::post('/search', 'ItemController@searchItem');

        // Labels Routes
        Route::get('/labels/categories/{category_id:[0-9]+}', 'ItemController@getCategoryItems');
        Route::get('/labels/suppliers/{supplier_id:[0-9]+}', 'ItemController@getSupplierItems');
        Route::delete('/variants/delete/{id:[0-9]+}', 'ItemController@deleteVariant');
        Route::post('/next-ski', 'ItemController@nextRef');

        // Labels Routes
        Route::get('/labels/categories/{category_id:[0-9]+}', 'ItemController@getCategoryItems');
        Route::get('/labels/suppliers/{supplier_id:[0-9]+}', 'ItemController@getSupplierItems');
        Route::delete('/variants/delete/{id:[0-9]+}', 'ItemController@deleteVariant');
        Route::post('/next-ski', 'ItemController@nextRef');

        // Labels Routes
        Route::get('/labels/categories/{category_id:[0-9]+}', 'ItemController@getCategoryItems');
        Route::get('/labels/suppliers/{supplier_id:[0-9]+}', 'ItemController@getSupplierItems');
    });

    // Supply Routes
    Route::group(['prefix' => 'supplies', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SupplyController@getSupplies');
        Route::get('/{id:[0-9]+}', 'SupplyController@getSupply');
        Route::post('/create', 'SupplyController@createSupplies');
        Route::post('/update/{id:[0-9]+}', 'SupplyController@updateSupplies');
    });

    //    Item Supply Routes
    Route::group(['prefix' => 'item_supplies', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ItemSupplyController@getItemSupplies');
        Route::get('/{id:[0-9]+}', 'ItemSupplyController@getItemSupplied');
        Route::post('/create', 'ItemSupplyController@createItemSupplied');
    });

    //    Supplier Payments Routes
    Route::group(['prefix' => 'supplier_payments', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SupplierPaymentController@getSupplierPayments');
        Route::get('/{id:[0-9]+}', 'SupplierPaymentController@getSupplierPayment');
        Route::post('/create', 'SupplierPaymentController@createSupplierPayment');
        Route::post('/update/{id:[0-9]+}', 'SupplierPaymentController@updateSupplierPayment');
    });

//    Warehouse Routes
    Route::group(['prefix' => 'warehouses', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'WarehouseController@all');
        Route::get('/{id:[0-9]+}', 'WarehouseController@getWarehouse');
        Route::post('/create', 'WarehouseController@createWarehouse');
        Route::post('/{id:[0-9]+}/update', 'WarehouseController@updateWarehouse');
        Route::post('/{id:[0-9]+}/update-price/{item_id:[0-9]+}', 'WarehouseController@updateItemSellingPrice');
        Route::post('/{id:[0-9]+}/archive', 'WarehouseController@archiveWarehouse');
        Route::post('/default/{id:[0-9]+}', 'WarehouseController@changeDefaultWarehouse');
        Route::get('/{id:[0-9]+}/items', 'WarehouseController@getWarehouseItems');
        Route::get('/{id:[0-9]+}/items-only', 'WarehouseController@getItems');
        Route::get('/{id:[0-9]+}/item', 'WarehouseController@getWarehousesItem');
        Route::get('/stock-item/{id:[0-9]+}', 'WarehouseController@getStockItem');
        Route::post('/adjust-stock-item/{id:[0-9]+}', 'WarehouseController@adjust_stock');
        Route::post('/adjust-warehouse-stock/{warehouse:[0-9]+}', 'WarehouseController@adjust_warehouse_stock');
        Route::get('/get-stock-history', 'WarehouseController@get_stock_history');
        Route::get('/get-stock-history-items/{id:[0-9]+}', 'WarehouseController@get_stock_history_items');
    });
    // Price List Routes
    Route::group(['prefix' => 'price_lists', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'PriceListController@getPriceLists');
        Route::post('/create', 'PriceListController@addPriceList');
        Route::post('/remove/{id:[0-9]+}', 'PriceListController@removePriceList');
    });
    // Config Routes
    Route::group(['prefix' => 'configs', 'middleware' => 'auth:api'], function () {
        Route::get('/default-messages', 'ConfigController@getDefaultMessages');
        Route::post('/default-messages/{id:[0-9]+}', 'ConfigController@setDefaultMessage');
    });
    // Customer Routes
    Route::group(['prefix' => 'customers', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'CustomerController@getCustomers');
        Route::post('/create', 'CustomerController@createCustomer');
        Route::post('/edit/{id:[0-9]+}', 'CustomerController@updateCustomer');
        Route::post('/archive/{id:[0-9]+}', 'CustomerController@archiveCustomer');
        Route::post('/archive/{id:[0-9]+}/restore', 'CustomerController@archiveCustomer');
        Route::get('/archives', 'CustomerController@getArchiveCustomers');
        Route::post('/update-credit-status/{id:[0-9]+}', 'CustomerController@updateCreditCustomer');
        Route::post('/price-list/{customer_id:[0-9]+}[/{pl_id:[0-9]+}]', 'CustomerController@updateCustomerPriceList');
    });

    // Account Routes
    Route::group(['prefix' => 'account', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'AccountController@getAccount');
        Route::get('/deposits', 'DepositController@getDeposits');
        Route::post('/deposits/create', 'DepositController@makeDeposit');
        Route::get('/withdraws', 'WithdrawalController@getWithdrawals');
        Route::post('/withdraws/create', 'WithdrawalController@makeWithdrawal');
        Route::get('/subcategories', 'AccountSubcategoryController@getAllSubcategories');

        // Account Categories Route
        Route::group(['prefix' => 'categories'], function () {
            Route::get('/', 'AccountCategoryController@getCategories');
            Route::post('/create', 'AccountCategoryController@createCategory');
            Route::post('/{id:[0-9]+}/update', 'AccountCategoryController@updateCategory');
            Route::get('/{id:[0-9]+}/summary', 'AccountCategoryController@getCategorySummary');
            Route::get('/{id:[0-9]+}/subcategories', 'AccountSubcategoryController@getSubcategories');
            Route::post('/{id:[0-9]+}/subcategories/create', 'AccountSubcategoryController@createSubcategory');
            Route::post('/subcategories/{sub_id}/update', 'AccountSubcategoryController@updateSubcategory');
        });
    });

    //    Employee Route
    Route::group(['prefix' => 'employee', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'EmployeeController@getEmployees');
        Route::get('/{id:[0-9]+}', 'EmployeeController@getEmployee');
        Route::post('/create', 'EmployeeController@createEmployee');
        Route::post('/{id:[0-9]+}/state', 'EmployeeController@changeState');
        Route::get('/archive/all', 'EmployeeController@getArchiveEmployees');
        Route::post('/archive/{id:[0-9]+}/restore', 'EmployeeController@changeState');
        Route::get('/{id:[0-9]+}/payments', 'EmployeeController@getEmployeePayments');
        Route::post('/{id:[0-9]+}/debts/cash-in', 'EmployeeController@createUnattendedDebt');
    });

    //    Employee Type Route
    Route::group(['prefix' => 'employee/type', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'EmployeeTypeController@getTypes');
        Route::get('/{id:[0-9]+}', 'EmployeeTypeController@getType');
        Route::post('/create', 'EmployeeTypeController@createType');
        Route::post('/{id:[0-9]+}/update', 'EmployeeTypeController@updateType');
    });

    //    Debt Route
    Route::group(['prefix' => 'debts', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'DebtController@getDebts');
        Route::get('/{id:[0-9]+}', 'DebtController@getDebt');
        Route::get('/employee/{id:[0-9]+}', 'DebtController@getEmployeeDebts');
        Route::get('/employee/{id:[0-9]+}/archive', 'DebtController@getEmployeeArchivedDebts');
        Route::post('/create', 'DebtController@createDebt');
        Route::post('/{id:[0-9]+}/state', 'DebtController@changeState');
        Route::get('/archive/all', 'DebtController@getArchiveDebts');
        Route::post('/{id:[0-9]+}/pay', 'DebtController@payDebt');
        Route::get('/payments', 'DebtController@getAllPayments');
        Route::get('/payments/{id:[0-9]+}/history', 'DebtController@getPaymentHistory');
        Route::post('/{id:[0-9]+}/delete', 'DebtController@deleteDebt');
    });

    //    DebtType Route
    Route::group(['prefix' => 'debts/type', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'DebtTypeController@getTypes');
        Route::get('/all', 'DebtTypeController@getAllTypes');
        Route::post('/create', 'DebtTypeController@createType');
        Route::get('/{id:[0-9]+}', 'DebtTypeController@getType');
        Route::post('{id:[0-9]+}/update', 'DebtTypeController@updateDebtType');
        Route::post('{id:[0-9]+}/archive', 'DebtTypeController@archiveDebtType');
    });

    //    Departments Route
    Route::group(['prefix' => 'departments', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'DepartmentController@getDepartments');
        Route::get('/mine', 'DepartmentController@getMyDepartments');
        Route::get('all', 'DepartmentController@getAllDepartments');
        Route::get('/contains-warehouse', 'DepartmentController@getDepartmentsWithWarehouse');
        Route::get('/contains-warehouse-item[/{item_id:[0-9]+}]', 'DepartmentController@getDepartmentsWithWarehouseItem');
        Route::post('/create', 'DepartmentController@createDepartment');
        Route::post('/{id:[0-9]+}/update', 'DepartmentController@updateDepartment');
        Route::post('/{id:[0-9]+}/archive', 'DepartmentController@toggleState');
        Route::post('/{id:[0-9]+}/warehouse/{wh:[0-9]+}', 'DepartmentController@setWarehouse');
        Route::post('/{id:[0-9]+}/settings', 'DepartmentController@updateDepartmentSettings');
        Route::get('/archive', 'DepartmentController@getArchivedDepartment');
    });

    //    Transfers Route
    Route::group(['prefix' => 'transfers', 'middleware' => 'auth:api'], function () {
        Route::post('/', 'TransferController@fetchTransfers');
        Route::post('/execute', 'TransferController@executeTransfers');
    });

    //    Employee Payments Route
    Route::group(['prefix' => 'employee/payments', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'EmployeePaymentController@getEmployeePayments');
        Route::get('/{id:[0-9]+}', 'EmployeePaymentController@getPayment');
        Route::get('/employee/{id:[0-9]+}', 'EmployeePaymentController@getEmployeePayment');
        Route::post('/create', 'EmployeePaymentController@createEmployeePayment');
    });

    //    Salary Route
    Route::group(['prefix' => 'salaries', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SalaryController@getSalaries');
        Route::get('/{id:[0-9]+}', 'SalaryController@getSalary');
        Route::get('/employee/{id:[0-9]+}', 'SalaryController@getEmployeeSalary');
        Route::post('/create', 'SalaryController@createEmployeeSalary');
    });

    Route::group(['prefix' => 'users', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'AdminController@index');
        Route::get('/cashiers', 'AdminController@getCashierUsers');
        Route::post('/create', 'AdminController@createUser');
        Route::post('/{id:[0-9]+}/update', 'AdminController@updateUser');
        Route::post('/{id:[0-9]+}/status', 'AdminController@changeStatus');
        Route::post('/{id:[0-9]+}/password-reset', 'AdminController@resetPassword');
    });

    $router->group(['prefix' => 'roles', 'middleware' => 'auth:api'], function () use ($router) {
        $router->get('/', 'RoleController@index');
        $router->get('/system', 'RoleController@systemRoles');
        $router->post('create', 'RoleController@store');
        $router->post('update/{id:[0-9]+}', 'RoleController@update');
        $router->post('delete/{id:[0-9]+}', 'RoleController@delete');
        $router->post('status', 'RoleController@changeStatus');
        $router->get('auth-user', 'RoleController@authRolePermissions');
        $router->get('user/{id:[0-9]+}', 'RoleController@userRoles');
        $router->post('user/assign/{id:[0-9]+}', 'RoleController@assignUserRoles');
        $router->get('permissions/{id:[0-9]+}', 'RoleController@rolePermissions');
        $router->post('permissions/assign/{id:[0-9]+}', 'RoleController@assignRolePermissions');
    });

    $router->group(['prefix' => 'sales-receipts', 'middleware' => 'auth:api'], function () use ($router) {
        $router->get('/', 'SalesReceiptController@index');
        $router->post('/store', 'SalesReceiptController@store');
        $router->post('/status/{id:[0-9]+}', 'SalesReceiptController@changeStatus');
        $router->post('/delete/{id:[0-9]+}', 'SalesReceiptController@delete');
    });

    $router->group(['prefix' => 'permissions', 'middleware' => 'auth:api'], function () use ($router) {
        $router->get('/', 'RoleController@permissions');
        $router->get('list', 'RoleController@userPermissions');
    });

    //    Categories Route
    Route::group(['prefix' => 'categories', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'CategoriesController@index');
        Route::get('list', 'CategoriesController@categories');
        Route::post('store', 'CategoriesController@store');
        Route::post('{id:[0-9]+}/update', 'CategoriesController@update');
        Route::delete('{id:[0-9]+}/delete', 'CategoriesController@delete');
        Route::post('{id:[0-9]+}/tracked', 'CategoriesController@updateTrackedStatus');
    });


    //    Invoice Route
    Route::group(['prefix' => 'invoices', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'InvoiceController@getInvoices');
        Route::get('cashier-session', 'InvoiceController@getCashierSessionInvoices');
        Route::post('store', 'InvoiceController@store');
        Route::post('update/{id:[0-9]+}', 'InvoiceController@update');
        Route::post('delete/{id:[0-9]+}', 'InvoiceController@delete');
    });

    //    Sales Route
    Route::group(['prefix' => 'sales', 'middleware' => 'auth:api'], function () {
        Route::post('/', 'SaleController@index');
        Route::post('/print', 'SaleController@printSales');
        Route::get('/{id:[0-9]+}', 'SaleController@getSale');
        // @todo Verify print works from server // done
        Route::post('/inventory/print', 'InvoiceController@printSessionReport');
    });

    //    Service Controller
    Route::group(['prefix' => 'services', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ServiceController@index');
        Route::get('/prospects', 'ServiceController@serviceProspects');
        Route::get('auth-department', 'ServiceController@getDepartmentServicesFirst');
        Route::get('/list', 'ServiceController@list');
        Route::post('store/{user_id:[0-9]+}', 'ServiceController@store');
        Route::post('status/{id:[0-9]+}', 'ServiceController@changeStatus');
        // Route::post('upload', 'ServiceController@upload');
        // Route::post('update/{id:[0-9]+}', 'ServiceController@update');
        // Route::post('delete/{id:[0-9]+}', 'ServiceController@delete');
        Route::post('{id:[0-9]+}/departments/{department_id:[0-9]+}', 'ServiceController@updateDepartment');

        Route::post('inventory', 'ServiceController@printServiceInventory');
        Route::post('query', 'ServiceController@query');
        Route::post('print-query', 'ServiceController@printServiceQuery');
    });

    //    Sessions Route
    Route::group(['prefix' => 'sessions', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SessionController@index');
        Route::get('/{id:[0-9]+}', 'SessionController@getDepartmentSessions');
        Route::get('/sessionable-users', 'SessionController@getSessionableUsers');
        Route::post('/store', 'SessionController@store');
        Route::post('/close/{id:[0-9]+}', 'SessionController@close');
        Route::post('/close', 'SessionController@cashier_close_session');
        // Session report
        Route::get('/report/generate', 'SessionController@generateSessionReport');
    });

    //    Sales Receipts Route
    Route::group(['prefix' => 'sales-receipts', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SalesReceiptController@index');
        Route::post('/store', 'SalesReceiptController@store');
        Route::post('/status/{id:[0-9]+}', 'SalesReceiptController@changeStatus');
        Route::post('/delete/{id:[0-9]+}', 'SalesReceiptController@delete');
    });

    //    Sales Order Route
    Route::group(['prefix' => 'sales_orders', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'SalesOrderController@index');
        Route::get('/session', 'SalesOrderController@getSessionSalesOrder');
        Route::post('/store', 'SalesOrderController@create');
        Route::get('/{id:[0-9]+}', 'SalesOrderController@getSaleOrder');
        Route::post('/update/{id:[0-9]+}', 'SalesOrderController@update');
        Route::post('/update-label/{id:[0-9]+}', 'SalesOrderController@updateTicketLabel');
        Route::post('/delete/{id:[0-9]+}', 'SalesOrderController@delete');
        Route::post('/revoke/{id:[0-9]+}', 'SalesOrderController@revoke');

        Route::post('/delete_orders', 'SalesOrderController@deleteOrders');
        Route::post('/merge_orders', 'SalesOrderController@mergeOrders');
        Route::post('/split_order', 'SalesOrderController@splitOrder');
        Route::post('/transfer_orders', 'SalesOrderController@transferOrders');
    });

    //    Sale Order Items Route
//    Route::group(['prefix' => 'sales-order-items', 'middleware' => 'auth:api'], function () {
//        Route::get('/', 'SaleOrderItemController@index');
//        Route::post('/create', 'SaleOrderItemController@create');
//        Route::get('/{id}', 'SaleOrderItemController@getSaleOrder');
//        Route::post('/update/{id}', 'SaleOrderItemController@update');
//        Route::post('/delete/{id}', 'SaleOrderItemController@delete');
//    });

    //    Payment Route
    Route::group(['prefix' => 'payment', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'PaymentController@getPayments');
        Route::get('/{id:[0-9]+}', 'PaymentController@getPayment');
        Route::post('/store', 'PaymentController@createPayment');
        Route::post('/status/{id}', 'PaymentController@changeStatus');
        Route::post('/delete/{id}', 'PaymentController@delete');
    });

    // PaymentMethod Route
    Route::group(['prefix' => 'payment_methods', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'PaymentMethodController@getPaymentMethods');
        Route::get('/active', 'PaymentMethodController@getActivePaymentMethods');
        Route::post('/{id:[0-9]+}/update', 'PaymentMethodController@updatePaymentMethod');
        Route::post('/store', 'PaymentMethodController@store');
        Route::post('/{id:[0-9]+}/status', 'PaymentMethodController@toggleStatus');
        Route::post('/{id:[0-9]+}/delete', 'PaymentMethodController@delete');
        Route::post('/{id:[0-9]+}/default', 'PaymentMethodController@setDefault');
    });

    // Modifiers Route
    Route::group(['prefix' => 'modifiers', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ModifierController@index');
        Route::post('/create', 'ModifierController@store');
        Route::post('/update/{id}', 'ModifierController@update');
        Route::delete('/delete/{id}', 'ModifierController@destroy');
        Route::get('/options', 'ModifierController@getOptions');
        Route::get('/options/{modifier_id}', 'ModifierController@getModifierOptions');
        Route::post('/options/create', 'ModifierController@storeOption');
        Route::post('/options/update/{id}', 'ModifierController@updateOption');
        Route::delete('/options/delete/{id}', 'ModifierController@destroyOption');
    });

    // Item Modifiers Route
    Route::group(['prefix' => 'item_modifiers', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ItemModifierController@index');
        Route::post('/create', 'ItemModifierController@store');
        Route::delete('/delete/{id}', 'ItemModifierController@destroy');
    });

    // Variants Route
    Route::group(['prefix' => 'variants', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'VariantController@index');
        Route::post('/create', 'VariantController@store');
        Route::post('/create/{item_id}', 'VariantController@create');
        Route::post('/update/{id}', 'VariantController@update');
        Route::get('/item/{item_id}', 'VariantController@get_item_variant');
        Route::delete('/delete/{id}', 'VariantController@destroy');
        Route::get('/options', 'VariantController@getOptions');
        Route::get('/options/{variant_id}', 'VariantController@getVariantOptions');
        Route::post('/options/update/{id}', 'VariantController@updateOption');
        Route::delete('/options/delete/{id}', 'VariantController@destroyOption');
        Route::post('/all/create/{item_id}', 'VariantController@storeVariantAndOptions');
    });

    // Item Variants Route
    Route::group(['prefix' => 'item_variants', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ItemVariantController@index');
        Route::get('/{item_id}', 'ItemVariantController@getItemVariants');
        Route::post('/create', 'ItemVariantController@store');
        Route::post('/update/{id}', 'ItemVariantController@update');
        Route::delete('/delete/{id}', 'ItemVariantController@destroy');
    });

    // Discounts Route
    Route::group(['prefix' => 'discounts', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'DiscountController@index');
        Route::post('/create', 'DiscountController@store');
        Route::post('/update/{id}', 'DiscountController@update');
        Route::delete('/delete/{id}', 'DiscountController@destroy');
    });

    // Taxes Route
    Route::group(['prefix' => 'taxes', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'TaxController@index');
        Route::post('/create', 'TaxController@store');
        Route::post('/update/{id}', 'TaxController@update');
        Route::delete('/delete/{id}', 'TaxController@destroy');
    });

    // Item Taxes Route
    Route::group(['prefix' => 'item_taxes', 'middleware' => 'auth:api'], function () {
        Route::get('/', 'ItemTaxController@index');
        Route::post('/create', 'ItemTaxController@store');
        Route::delete('/delete/{id}', 'ItemTaxController@destroy');
    });

//    Print Route
    Route::group(['prefix' => 'print'], function () {
        Route::get('/{id}', 'PrintController@printSalesOrder');
        Route::get('/sale/{id}', 'PrintController@printSale');

        // Print session report
        Route::post('/session/sale', function (\Illuminate\Http\Request $request) {
            return response()->json([
                'sales' => App\Sale::with(['items.item.category', 'service', 'payments'])->whereIn('session_id', $request->sessions)->get(),
                'sessions' => App\Session::with('user')->whereIn('id', $request->sessions)->get()
            ]);
        });
        Route::get('/print/config', function () {
            return App\Config::latest()->first();
        });
        Route::post('/session/items', function (\Illuminate\Http\Request $request) {
            \Illuminate\Support\Facades\Log::debug(json_encode($request->input('item_ids')));
            return App\Item::whereIn('id', $request->item_ids)
                ->select('id', 'name', 'category_id')
                ->with(['category' => function ($q) {
                    return $q->select('id', 'name');
                }])->get();
        });
        Route::post('/session/invoices', function (\Illuminate\Http\Request $request) {
            // \Illuminate\Support\Facades\Log::debug(json_encode($data));
            return App\Invoice::with('item')->whereIn('session_id', $request->sessions)->whereItemId($request->item_id)->get();
        });
        Route::get('/session/services', function (\Illuminate\Http\Request $request) {
            return App\Session::with(['sales.service'])->whereIn('id', $request->sessions)->get();
        });

    });

});



